@@ -29,11 +29,14 @@ typedef enum   _GtkHTMLEditPropertyType       GtkHTMLEditPropertyType;
 typedef struct _GtkHTMLEditPropertiesDialog   GtkHTMLEditPropertiesDialog;
 
 #include <gtkhtml.h>
+#include <bonobo.h>
 #include "search.h"
 #include "replace.h"
 #include "image.h"
 #include "link.h"
 #include "rule.h"
+#include "engine.h"
+#include "Spell.h"
 
 struct _GtkHTMLControlData {
 	GtkHTML   *html;
@@ -46,10 +49,6 @@ struct _GtkHTMLControlData {
 	GtkHTMLSearchDialog     *search_dialog;
 	GtkHTMLReplaceDialog    *replace_dialog;
 
-	/* objects dialogs */
-	GtkHTMLImageDialog      *image_dialog;
-	GtkHTMLRuleDialog       *rule_dialog;
-
 	/* object from last button press event */
 	HTMLObject *obj;
 
@@ -72,6 +71,10 @@ struct _GtkHTMLControlData {
 
 	guint font_style_changed_connection_id;
 	gboolean block_font_style_change;
+
+	GNOME_Spell_Dictionary  dict;
+	BonoboObjectClient     *dict_client;
+	EditorEngine       *editor_bonobo_engine;
 };
 
 GtkHTMLControlData * gtk_html_control_data_new       (GtkHTML *html, GtkWidget *vbox);
