@@ -32,11 +32,18 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#include "HTMLEditor.h"
+#include <gal/widgets/e-scroll-frame.h>
+
+#include "Editor.h"
 
 #include "gtkhtml.h"
+#include "htmlcursor.h"
+#include "htmlengine.h"
 #include "htmlengine-edit.h"
+#include "htmlengine-edit-selection-updater.h"
+#include "htmlselection.h"
 
+#include "engine.h"
 #include "menubar.h"
 #include "persist-file-impl.h"
 #include "persist-stream-impl.h"
@@ -54,7 +61,7 @@
 #include "gtkhtmldebug.h"
 
 #ifdef USING_OAF
-#define CONTROL_FACTORY_ID "OAFIID:control-factory:html-editor:97cebd11-9e8c-45c3-9a0a-5a269c760c00"
+#define CONTROL_FACTORY_ID "OAFIID:GNOME_GtkHTML_EditorFactory"
 #else
 #define CONTROL_FACTORY_ID "control-factory:html-editor"
 #endif
@@ -69,6 +76,8 @@ struct _SetFrameData {
 };
 typedef struct _SetFrameData SetFrameData;
 
+GtkHTMLEditorAPI *editor_api;
+
 static BonoboGenericFactory *factory = NULL;
 static gint active_controls = 0;
 
@@ -80,7 +89,7 @@ set_frame_cb (BonoboControl *control,
 	BonoboUIComponent *ui_component;
 	GtkHTMLControlData *control_data;
 	GtkWidget *toolbar;
-	GtkWidget *scrolled_window;
+	GtkWidget *scroll_frame;
 
 	control_data = (GtkHTMLControlData *) data;
 
@@ -93,17 +102,17 @@ set_frame_cb (BonoboControl *control,
 	toolbar = toolbar_style (control_data);
 	gtk_box_pack_start (GTK_BOX (control_data->vbox), toolbar, FALSE, FALSE, 0);
 
-	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_container_add (GTK_CONTAINER (scrolled_window), GTK_WIDGET (control_data->html));
-	gtk_widget_show_all (scrolled_window);
+	scroll_frame = e_scroll_frame_new (NULL, NULL);
+	e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (scroll_frame), GTK_SHADOW_IN);
+	e_scroll_frame_set_policy (E_SCROLL_FRAME (scroll_frame), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+	gtk_container_add (GTK_CONTAINER (scroll_frame), GTK_WIDGET (control_data->html));
+	gtk_widget_show_all (scroll_frame);
 
-	gtk_box_pack_start (GTK_BOX (control_data->vbox), scrolled_window, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (control_data->vbox), scroll_frame, TRUE, TRUE, 0);
 
 	/* Setup the menu bar.  */
 
 	menubar_setup (ui_component, control_data);
-	toolbar_setup (ui_component, control_data);
 }
 
 static gint
@@ -118,6 +127,7 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 		case HTML_TYPE_IMAGE:
 		case HTML_TYPE_LINKTEXTMASTER:
 		case HTML_TYPE_TEXTMASTER:
+		case HTML_TYPE_RULE:
 			run_dialog = TRUE;
 			break;
 		default:
@@ -152,6 +162,14 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 					: GTK_HTML_EDIT_PROPERTY_LINK;
 						
 				break;
+			case HTML_TYPE_RULE:
+				gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+									   GTK_HTML_EDIT_PROPERTY_RULE, _("Rule"),
+									   rule_properties,
+									   rule_apply_cb,
+									   rule_close_cb);
+				start = GTK_HTML_EDIT_PROPERTY_RULE;
+				break;
 			default:
 			}
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
@@ -221,7 +239,7 @@ load_from_corba (BonoboControl *control,
 		 const char *url, 
 		 GtkHTMLStream *handle)
 {
-	HTMLEditor_Resolver resolver;
+	GNOME_GtkHTML_Editor_Resolver resolver;
 	Bonobo_ControlFrame Frame;
 	CORBA_Environment ev;	
 	int ret_val = FALSE;
@@ -230,8 +248,8 @@ load_from_corba (BonoboControl *control,
 
 	Frame = bonobo_control_get_control_frame (control);
 	if (Frame != CORBA_OBJECT_NIL) {
-		resolver = Bonobo_Unknown_query_interface (Frame,
-							   "IDL:HTMLEditor/Resolver:1.0",
+		resolver = Bonobo_Unknown_queryInterface (Frame,
+							   "IDL:GNOME/HTMLEditor/Resolver:1.0",
 							   &ev);
 		
 		if (resolver != CORBA_OBJECT_NIL) {
@@ -242,7 +260,7 @@ load_from_corba (BonoboControl *control,
 
 			sink = bonobo_object_corba_objref (object);
 			
-			HTMLEditor_Resolver_loadURL (resolver, sink, url, &ev);
+			GNOME_GtkHTML_Editor_Resolver_loadURL (resolver, sink, url, &ev);
 			if (ev._major != CORBA_NO_EXCEPTION){
 				g_warning ("Corba load exception");
 			} else {
@@ -312,32 +330,72 @@ static void
 destroy_control_data_cb (GtkObject *control, GtkHTMLControlData *cd)
 {
 	gtk_html_control_data_destroy (cd);
-
-	printf ("active--\n");
 	active_controls --;
 
-	printf ("control_destroy %d\n", active_controls);
-
 	if (active_controls)
 		return;
+
 	bonobo_object_unref (BONOBO_OBJECT (factory));
 	gtk_main_quit ();
 }
 
+static void
+editor_control_construct (BonoboControl *control, GtkWidget *vbox)
+{
+	GtkHTMLControlData *control_data;
+	BonoboPersistStream *stream_impl;
+	BonoboPersistFile *file_impl;
+	GtkWidget *html_widget;
+
+	active_controls++;
+
+	/* GtkHTML widget */
+
+	html_widget = gtk_html_new ();
+	gtk_html_load_empty (GTK_HTML (html_widget));
+	gtk_html_set_editable (GTK_HTML (html_widget), TRUE);
+
+	control_data = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
+	gtk_html_set_editor_api (GTK_HTML (html_widget), editor_api, control_data);
+
+	/* HTMLEditor::Engine */
+
+	control_data->editor_bonobo_engine = editor_engine_new (GTK_HTML (html_widget));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (control_data->editor_bonobo_engine));
+
+	/* Bonobo::PersistStream */
+
+	stream_impl = persist_stream_impl_new (GTK_HTML (html_widget));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (stream_impl));
+
+	/* Bonobo::PersistFile */
+
+	file_impl = persist_file_impl_new (GTK_HTML (html_widget));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (file_impl));
+
+	/* Part of the initialization must be done after the control is
+	   embedded in its control frame.  We use the "set_frame" signal to
+	   handle that.  */
+
+	gtk_signal_connect (GTK_OBJECT (control), "set_frame",
+			    GTK_SIGNAL_FUNC (set_frame_cb), control_data);
+
+	gtk_signal_connect (GTK_OBJECT (control), "destroy",
+			    GTK_SIGNAL_FUNC (destroy_control_data_cb), control_data);
+
+	gtk_signal_connect (GTK_OBJECT (html_widget), "url_requested",
+			    GTK_SIGNAL_FUNC (url_requested_cb), control);
+
+	gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
+			    GTK_SIGNAL_FUNC (html_button_pressed), control_data);
+}
+
 static BonoboObject *
 editor_control_factory (BonoboGenericFactory *factory,
 			gpointer closure)
 {
 	BonoboControl *control;
-	BonoboPersistStream *stream_impl;
-	BonoboPersistFile *file_impl;
-	GtkWidget *html_widget;
 	GtkWidget *vbox;
-	GtkHTMLControlData *control_data;
-
-	html_widget = gtk_html_new ();
-	gtk_html_load_empty (GTK_HTML (html_widget));
-	gtk_html_set_editable (GTK_HTML (html_widget), TRUE);
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
@@ -345,59 +403,146 @@ editor_control_factory (BonoboGenericFactory *factory,
 	/* g_warning ("Creating a new GtkHTML editor control."); */
 	control = bonobo_control_new (vbox);
 
-	if (control) {
-
-		active_controls++;
-		printf ("active++\n");
-
-		/* Bonobo::PersistStream */
-
-		stream_impl = persist_stream_impl_new (GTK_HTML (html_widget));
-		bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (stream_impl));
-
-		/* Bonobo::PersistFile */
-
-		file_impl = persist_file_impl_new (GTK_HTML (html_widget));
-		bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (file_impl));
-
-		/* Part of the initialization must be done after the control is
-		   embedded in its control frame.  We use the "set_frame" signal to
-		   handle that.  */
-
-		control_data = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
-
-		gtk_signal_connect (GTK_OBJECT (control), "set_frame",
-				    GTK_SIGNAL_FUNC (set_frame_cb), control_data);
-
-		gtk_signal_connect (GTK_OBJECT (control), "destroy",
-				    GTK_SIGNAL_FUNC (destroy_control_data_cb), control_data);
-
-		gtk_signal_connect (GTK_OBJECT (html_widget), "url_requested",
-				    GTK_SIGNAL_FUNC (url_requested_cb), control);
-
-		gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
-				    GTK_SIGNAL_FUNC (html_button_pressed), control_data);
-
-#ifdef GTKHTML_HAVE_PSPELL
-		gtk_signal_connect (GTK_OBJECT (html_widget), "spell_suggestion_request",
-				    GTK_SIGNAL_FUNC (spell_suggestion_request_cb), control_data);
-#endif
-	}
+	if (control)
+		editor_control_construct (control, vbox);
+	else
+		gtk_widget_unref (vbox);
 
 	return BONOBO_OBJECT (control);
 }
 
+static gboolean
+editor_api_command (GtkHTML *html, GtkHTMLCommandType com_type, gpointer data)
+{
+	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
+	gboolean rv = TRUE;
+
+	switch (com_type) {
+	case GTK_HTML_COMMAND_POPUP_MENU:
+		popup_show (cd, NULL);
+		break;
+	case GTK_HTML_COMMAND_PROPERTIES_DIALOG:
+		property_dialog_show (cd);
+		break;
+	default:
+		rv = FALSE;
+	}
+
+	return rv;
+}
+
+static GtkType
+wait_for_bonobo_patch__bonobo_arg_type_to_gtk (BonoboArgType id)
+{
+	CORBA_Environment ev;
+	GtkType gtk_type = GTK_TYPE_NONE;
+
+	CORBA_exception_init (&ev);
+
+	if (bonobo_arg_type_is_equal (TC_char, id, &ev))         gtk_type = GTK_TYPE_CHAR;
+	else if (bonobo_arg_type_is_equal (TC_boolean, id, &ev)) gtk_type = GTK_TYPE_BOOL;
+	else if (bonobo_arg_type_is_equal (TC_short,   id, &ev)) gtk_type = GTK_TYPE_INT;
+	else if (bonobo_arg_type_is_equal (TC_ushort,  id, &ev)) gtk_type = GTK_TYPE_UINT;
+	else if (bonobo_arg_type_is_equal (TC_long,    id, &ev)) gtk_type = GTK_TYPE_LONG;
+	else if (bonobo_arg_type_is_equal (TC_ulong,   id, &ev)) gtk_type = GTK_TYPE_ULONG;
+	else if (bonobo_arg_type_is_equal (TC_float,   id, &ev)) gtk_type = GTK_TYPE_FLOAT;
+	else if (bonobo_arg_type_is_equal (TC_double,  id, &ev)) gtk_type = GTK_TYPE_DOUBLE;
+	else if (bonobo_arg_type_is_equal (TC_string,  id, &ev)) gtk_type = GTK_TYPE_STRING;
+	else
+		g_warning ("Unmapped bonobo arg type");
+
+	CORBA_exception_free (&ev);
+
+	return gtk_type;
+}
+
+static GtkArg *
+editor_api_event (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **args, gpointer data)
+{
+	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
+	GtkArg *gtk_retval = NULL;
+
+	/* printf ("editor_api_event\n"); */
+
+	if (event_type != GTK_HTML_EDITOR_EVENT_COMMAND && event_type != GTK_HTML_EDITOR_EVENT_IMAGE_URL) {
+		g_warning ("Now, only COMMAND and IMAGE_URL events are supported.\n");
+		return NULL;
+	}
+	if (cd->editor_bonobo_engine) {
+		GNOME_GtkHTML_Editor_Engine engine;
+		GNOME_GtkHTML_Editor_Listener listener;
+		CORBA_Environment ev;
+
+		engine = bonobo_object_corba_objref (BONOBO_OBJECT (cd->editor_bonobo_engine));
+		if (engine != CORBA_OBJECT_NIL
+		    && (listener = GNOME_GtkHTML_Editor_Engine__get_listener (engine, &ev)) != CORBA_OBJECT_NIL) {
+
+			BonoboArg *arg = bonobo_arg_new (bonobo_arg_type_from_gtk (args [0]->type));
+			BonoboArg *bonobo_retval;
+
+			bonobo_arg_from_gtk (arg, args [0]);
+
+			/* printf ("sending to listener\n"); */
+			CORBA_exception_init (&ev);
+			bonobo_retval = GNOME_GtkHTML_Editor_Listener_event (listener,
+									 event_type ==  GTK_HTML_EDITOR_EVENT_COMMAND
+									 ? "command" : "image_url",
+									 arg, &ev);
+			CORBA_free (arg);
+
+			if (ev._major == CORBA_NO_EXCEPTION) {
+				if (!bonobo_arg_type_is_equal (bonobo_retval->_type, TC_null, &ev)
+				    && !bonobo_arg_type_is_equal (bonobo_retval->_type, TC_void, &ev)) {
+					gtk_retval = gtk_arg_new (wait_for_bonobo_patch__bonobo_arg_type_to_gtk (bonobo_retval->_type));
+					bonobo_arg_to_gtk (gtk_retval, bonobo_retval);
+				}
+				CORBA_free (bonobo_retval);
+			}
+			CORBA_exception_free (&ev);
+		}
+	}
+	return gtk_retval;
+}
+
+static GtkEntry *
+editor_api_create_input_line (GtkHTML *html, gpointer data)
+{
+	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
+	GtkWidget *entry;
+
+	entry = gtk_entry_new ();
+	gtk_box_pack_end (GTK_BOX (cd->vbox), entry, FALSE, FALSE, 0);
+	gtk_widget_show (entry);
+
+	return GTK_ENTRY (entry);
+}
+
+static void
+new_editor_api ()
+{
+	editor_api = g_new (GtkHTMLEditorAPI, 1);
+
+	editor_api->check_word         = spell_check_word;
+	editor_api->suggestion_request = spell_suggestion_request;
+	editor_api->add_to_personal    = spell_add_to_personal;
+	editor_api->add_to_session     = spell_add_to_session;
+	editor_api->command            = editor_api_command;
+	editor_api->event              = editor_api_event;
+	editor_api->create_input_line  = editor_api_create_input_line;
+}
+
 void
 editor_control_factory_init (void)
 {
 	if (factory != NULL)
 		return;
 
+	new_editor_api ();
 	gdk_rgb_init ();
 	factory = bonobo_generic_factory_new (CONTROL_FACTORY_ID,
 					      editor_control_factory,
 					      NULL);
 
 	if (factory == NULL)
-		g_error ("I could not register the html-editor-control factory.");
+		g_error ("I could not register the GNOME_GtkHTML_Editor factory.");
 }
