@@ -21,6 +21,7 @@
     Author: Ettore Perazzoli <ettore@helixcode.com>
 */
 
+#include <config.h>
 #include <gnome.h>
 #include <bonobo.h>
 
@@ -131,7 +132,18 @@ insert_link_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cnam
 static void
 insert_rule_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cname)
 {
-	rule_insert (cd);
+	if (cd->properties_dialog)
+		gtk_html_edit_properties_dialog_close (cd->properties_dialog);
+
+	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, TRUE, _("Insert"));
+
+	gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+						   GTK_HTML_EDIT_PROPERTY_RULE, _("Rule"),
+						   rule_insert,
+						   rule_insert_cb,
+						   rule_close_cb);
+
+	gtk_html_edit_properties_dialog_show (cd->properties_dialog);
 }
 
 static void
@@ -157,7 +169,7 @@ BonoboUIVerb verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("InsertImage", insert_image_cb),
 	BONOBO_UI_UNSAFE_VERB ("InsertLink", insert_link_cb),
 	BONOBO_UI_UNSAFE_VERB ("InsertRule", insert_rule_cb),
-	
+
 	BONOBO_UI_VERB_END
 };
 
@@ -172,6 +184,6 @@ menubar_setup (BonoboUIComponent  *uic,
 	bonobo_ui_component_add_verb_list_with_data (uic, verbs, cd);
 
 	bonobo_ui_util_set_ui (uic, GNOMEDATADIR,
-			       "html-editor-control.xml",
-			       "html-editor-control");
+			       "GNOME_GtkHTML_Editor.xml",
+			       "GNOME_GtkHTML_Editor");
 }
