@@ -20,14 +20,20 @@
     Boston, MA 02111-1307, USA.
 */
 
-#include <config.h>
-#include <htmlengine.h>
-#include <htmllinktextmaster.h>
-#include <htmlengine-edit-copy.h>
-#include <htmlengine-edit-cut.h>
-#include <htmlengine-edit-paste.h>
-#include <htmlengine-edit-insert.h>
+#include "gtkhtml.h"
+#include "htmlcursor.h"
+#include "htmlengine.h"
+#include "htmllinktextmaster.h"
+#include "htmlengine-edit-copy.h"
+#include "htmlengine-edit-cut.h"
+#include "htmlengine-edit-paste.h"
+#include "htmlengine-edit-insert.h"
+#include "htmlimage.h"
+#include "htmlselection.h"
+
+#include "config.h"
 #include "popup.h"
+#include "spell.h"
 #include "properties.h"
 #include "paragraph.h"
 #include "image.h"
@@ -56,17 +62,17 @@ paste (GtkWidget *mi, GtkHTMLControlData *cd)
 static void
 remove_link (GtkWidget *mi, GtkHTMLControlData *cd)
 {
-	if (cd->html->engine->active_selection)
+	if (html_engine_is_selection_active (cd->html->engine))
 		html_engine_remove_link (cd->html->engine);
         else
 		html_engine_remove_link_object (cd->html->engine, cd->html->engine->cursor->object);
 }
 
 static void
-prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
+show_prop_dialog (GtkHTMLControlData *cd, GtkHTMLEditPropertyType start)
 {
-	GList *cur;
 	GtkHTMLEditPropertyType t;
+	GList *cur;
 
 	cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE, _("Properties"));
 
@@ -101,35 +107,45 @@ prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
 								   link_properties,
 								   link_apply_cb,
 								   link_close_cb);
-								   break;
+			break;
 		case GTK_HTML_EDIT_PROPERTY_BODY:
 			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
 								   t, _("Page"),
 								   body_properties,
 								   body_apply_cb,
 								   body_close_cb);
-								   break;
+			break;
+		case GTK_HTML_EDIT_PROPERTY_RULE:
+			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+								   t, _("Rule"),
+								   rule_properties,
+								   rule_apply_cb,
+								   rule_close_cb);
+			break;
 		}
 		cur = cur->next;
 	}
 
-	t = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), "type"));
-
 	gtk_html_edit_properties_dialog_show (cd->properties_dialog);
-	if (t >= 0)
-		gtk_html_edit_properties_dialog_set_page (cd->properties_dialog, t);
+	if (start >= 0)
+		gtk_html_edit_properties_dialog_set_page (cd->properties_dialog, start);
+}
+
+static void
+prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
+{
+	show_prop_dialog (cd, GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), "type")));
 }
 
-#ifdef GTKHTML_HAVE_PSPELL
 static void
 spell_suggest (GtkWidget *mi, GtkHTMLControlData *cd)
 {
 	HTMLEngine *e = cd->html->engine;
 
-	gtk_signal_emit_by_name (GTK_OBJECT (cd->html), "spell_suggestion_request",
-				 e->spell_checker, html_engine_get_word (e));
+	/* gtk_signal_emit_by_name (GTK_OBJECT (cd->html), "spell_suggestion_request",
+	   e->spell_checker, html_engine_get_word (e)); */
+	spell_suggestion_request (cd->html, html_engine_get_word (e), cd);
 }
-#endif
 
 #define ADD_ITEM(l,f,t) \
 		menuitem = gtk_menu_item_new_with_label (_(l)); \
@@ -137,7 +153,7 @@ spell_suggest (GtkWidget *mi, GtkHTMLControlData *cd)
 		gtk_menu_append (GTK_MENU (menu), menuitem); \
 		gtk_widget_show (menuitem); \
 		gtk_signal_connect (GTK_OBJECT (menuitem), "activate", GTK_SIGNAL_FUNC (f), cd); \
-                items++; items_sep++
+                (*items)++; items_sep++
 
 #define ADD_SEP \
         if (items_sep) { \
@@ -150,26 +166,29 @@ spell_suggest (GtkWidget *mi, GtkHTMLControlData *cd)
 #define ADD_PROP(x) \
         cd->properties_types = g_list_append (cd->properties_types, GINT_TO_POINTER (GTK_HTML_EDIT_PROPERTY_ ## x))
 
-gint
-popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
+static GtkWidget *
+prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 {
 	HTMLEngine *e = cd->html->engine;
 	HTMLObject *obj;
 	GtkWidget *menu;
 	GtkWidget *menuitem;
-	guint items = 0;
 	guint items_sep = 0;
 
 	obj  = cd->html->engine->cursor->object;
 	menu = gtk_menu_new ();
 
-#ifdef GTKHTML_HAVE_PSPELL
-	if (!e->active_selection && obj && html_object_is_text (obj) && !html_engine_word_is_valid (e)) {
+	if (cd->properties_types) {
+		g_list_free (cd->properties_types);
+		cd->properties_types = NULL;
+	}
+
+	if (!html_engine_is_selection_active (e) && obj && html_object_is_text (obj) && !html_engine_word_is_valid (e)) {
 		ADD_SEP;
 		ADD_ITEM ("Suggest word", spell_suggest, -1);
 	}
-#endif
-	if (e->active_selection
+
+	if (html_engine_is_selection_active (e)
 	    || (obj
 		&& (HTML_OBJECT_TYPE (obj) == HTML_TYPE_LINKTEXTMASTER
 		    || (HTML_OBJECT_TYPE (obj) == HTML_TYPE_IMAGE
@@ -178,18 +197,18 @@ popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
 		    ADD_ITEM (_("Remove link"), remove_link, -1);
 	}
 
-	if (e->active_selection) {
+	if (html_engine_is_selection_active (e)) {
 		ADD_SEP;
 		ADD_ITEM ("Copy", copy, -1);
 		ADD_ITEM ("Cut",  cut, -1);
 	}
 	if (e->cut_buffer) {
-		if (!e->active_selection) {
+		if (!html_engine_is_selection_active (e)) {
 			ADD_SEP;
 		}
 		ADD_ITEM ("Paste",  paste, -1);
 	}
-	if (e->active_selection) {
+	if (html_engine_is_selection_active (e)) {
 		ADD_SEP;
 		ADD_ITEM ("Text...", prop_dialog, GTK_HTML_EDIT_PROPERTY_TEXT);
 		ADD_PROP (TEXT);
@@ -200,6 +219,9 @@ popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
 	} else if (obj) {
 		switch (HTML_OBJECT_TYPE (obj)) {
 		case HTML_TYPE_RULE:
+			ADD_SEP;
+			ADD_ITEM ("Rule...", prop_dialog, GTK_HTML_EDIT_PROPERTY_RULE);
+			ADD_PROP (RULE);
 			break;
 		case HTML_TYPE_IMAGE:
 			ADD_SEP;
@@ -229,10 +251,31 @@ popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
 	ADD_ITEM ("Page...", prop_dialog, GTK_HTML_EDIT_PROPERTY_BODY);
 
 	gtk_widget_show (menu);
+
+	return menu;
+}
+
+gint
+popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
+{
+	GtkWidget *menu;
+	guint items;
+
+	menu = prepare_properties_and_menu (cd, &items);
 	if (items)
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
-				event->button, event->time);
+				event ? event->button : 0, event ? event->time : 0);
 	gtk_widget_unref (menu);
 
 	return (items > 0);
 }
+
+void
+property_dialog_show (GtkHTMLControlData *cd)
+{
+	guint items;
+
+	gtk_widget_unref (prepare_properties_and_menu (cd, &items));
+	if (items)
+		show_prop_dialog (cd, -1);
+}
