@@ -23,14 +23,17 @@
 
 /* FIXME: Should use BonoboUIHandler.  */
 
+#include <config.h>
 #include <gnome.h>
 #include <bonobo.h>
+#include <gal/widgets/widget-color-combo.h>
 
 #include "toolbar.h"
 #include "utils.h"
-#include "gtk-combo-box.h"
 #include "htmlcolor.h"
+#include "htmlcolorset.h"
 #include "htmlengine-edit-fontstyle.h"
+#include "htmlsettings.h"
 
 
 #define EDITOR_TOOLBAR_PATH "/HTMLEditor"
@@ -42,18 +45,18 @@ static struct {
 	GtkHTMLParagraphStyle style;
 	const gchar *description;
 } paragraph_style_items[] = {
-	{ GTK_HTML_PARAGRAPH_STYLE_NORMAL, _("Normal") },
-	{ GTK_HTML_PARAGRAPH_STYLE_H1, _("Header 1") },
-	{ GTK_HTML_PARAGRAPH_STYLE_H2, _("Header 2") },
-	{ GTK_HTML_PARAGRAPH_STYLE_H3, _("Header 3") },
-	{ GTK_HTML_PARAGRAPH_STYLE_H4, _("Header 4") },
-	{ GTK_HTML_PARAGRAPH_STYLE_H5, _("Header 5") },
-	{ GTK_HTML_PARAGRAPH_STYLE_H6, _("Header 6") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ADDRESS, _("Address") },
-	{ GTK_HTML_PARAGRAPH_STYLE_PRE, _("Pre") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT, _("List item (digit)") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED, _("List item (unnumbered)") },
-	{ GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN, _("List item (roman)") },
+	{ GTK_HTML_PARAGRAPH_STYLE_NORMAL, N_("Normal") },
+	{ GTK_HTML_PARAGRAPH_STYLE_H1, N_("Header 1") },
+	{ GTK_HTML_PARAGRAPH_STYLE_H2, N_("Header 2") },
+	{ GTK_HTML_PARAGRAPH_STYLE_H3, N_("Header 3") },
+	{ GTK_HTML_PARAGRAPH_STYLE_H4, N_("Header 4") },
+	{ GTK_HTML_PARAGRAPH_STYLE_H5, N_("Header 5") },
+	{ GTK_HTML_PARAGRAPH_STYLE_H6, N_("Header 6") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ADDRESS, N_("Address") },
+	{ GTK_HTML_PARAGRAPH_STYLE_PRE, N_("Pre") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT, N_("List item (digit)") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED, N_("List item (unnumbered)") },
+	{ GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN, N_("List item (roman)") },
 	{ GTK_HTML_PARAGRAPH_STYLE_NORMAL, NULL },
 };
 
@@ -105,7 +108,7 @@ setup_paragraph_style_option_menu (GtkHTML *html)
 	for (i = 0; paragraph_style_items[i].description != NULL; i++) {
 		GtkWidget *menu_item;
 
-		menu_item = gtk_menu_item_new_with_label (paragraph_style_items[i].description);
+		menu_item = gtk_menu_item_new_with_label (_(paragraph_style_items[i].description));
 		gtk_widget_show (menu_item);
 
 		gtk_menu_append (GTK_MENU (menu), menu_item);
@@ -157,6 +160,7 @@ setup_font_size_option_menu (GtkHTMLControlData *cd)
 	gchar size [3];
 
 	cd->font_size_menu = option_menu = gtk_option_menu_new ();
+
 	menu = gtk_menu_new ();
 	size [2] = 0;
 
@@ -189,87 +193,30 @@ setup_font_size_option_menu (GtkHTMLControlData *cd)
 }
 
 static void
-set_color (GtkWidget *w, gushort r, gushort g, gushort b, gushort a, GtkHTMLControlData *cd)
+color_changed (GtkWidget *w, GdkColor *gdk_color, GtkHTMLControlData *cd)
 {
-	HTMLColor *color = html_color_new_from_rgb (r, g, b);
+	HTMLColor *color = gdk_color
+		&& gdk_color != &html_colorset_get_color (cd->html->engine->settings->color_set, HTMLTextColor)->color
+		? html_color_new_from_gdk_color (gdk_color) : NULL;
 
 	html_engine_set_color (cd->html->engine, color);
-	html_color_unref (color);
-}
-
-static void
-color_button_clicked (GtkWidget *w, GtkHTMLControlData *cd)
-{
-	gdouble r, g, b, rn, gn, bn;
-	gtk_combo_box_popup_hide (GTK_COMBO_BOX (cd->combo));
-
-	gnome_color_picker_get_d (GNOME_COLOR_PICKER (cd->cpicker), &r, &g, &b, NULL);
-
-	rn = ((gdouble) w->style->bg [GTK_STATE_NORMAL].red)  /0xffff;
-	gn = ((gdouble) w->style->bg [GTK_STATE_NORMAL].green)/0xffff;
-	bn = ((gdouble) w->style->bg [GTK_STATE_NORMAL].blue) /0xffff;
-
-	if (r != rn || g != gn || b != bn) {
-		gnome_color_picker_set_d (GNOME_COLOR_PICKER (cd->cpicker), rn, gn, bn, 1.0);
-		set_color (cd->cpicker, rn*0xffff, gn*0xffff, bn*0xffff, 0xffff, cd);
-	}
-}
-
-static void
-default_clicked (GtkWidget *w, GtkHTMLControlData *cd)
-{
-	gtk_combo_box_popup_hide (GTK_COMBO_BOX (cd->combo));
-	gnome_color_picker_set_d (GNOME_COLOR_PICKER (cd->cpicker), 0.0, 0.0, 0.0, 1.0);
-	html_engine_set_color (cd->html->engine,
-			       html_colorset_get_color (cd->html->engine->settings->color_set, HTMLTextColor));
-	
+	if (color)
+		html_color_unref (color);
 }
 
 static GtkWidget *
 setup_color_combo (GtkHTMLControlData *cd)
 {
-       	GtkWidget *vbox, *table, *button;
-
-	cd->cpicker = gnome_color_picker_new ();
-	GTK_WIDGET_UNSET_FLAGS (cd->cpicker, GTK_CAN_FOCUS);
-	vbox = gtk_vbox_new (FALSE, 2);
-	button = gtk_button_new_with_label (_("Default"));
-	gtk_signal_connect (GTK_OBJECT (button), "clicked", default_clicked, cd);
-	gtk_signal_connect (GTK_OBJECT (cd->cpicker), "color_set", GTK_SIGNAL_FUNC (set_color), cd);
-	table = color_table_new (color_button_clicked, cd);
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), button);
-	gtk_box_pack_start_defaults (GTK_BOX (vbox), table);
-	cd->combo = gtk_combo_box_new (cd->cpicker, vbox);
-	if (!gnome_preferences_get_toolbar_relief_btn ()) {
-		gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (cd->combo), GTK_RELIEF_NONE);
-		gtk_button_set_relief (GTK_BUTTON (cd->cpicker), GTK_RELIEF_NONE);
-	}
+	cd->combo = color_combo_new (NULL, _("Automatic"),
+				     &html_colorset_get_color (cd->html->engine->settings->color_set,
+							       HTMLTextColor)->color, "toolbar_text");
+	GTK_WIDGET_UNSET_FLAGS (cd->combo, GTK_CAN_FOCUS);
+        gtk_signal_connect (GTK_OBJECT (cd->combo), "changed", GTK_SIGNAL_FUNC (color_changed), cd);
 
 	gtk_widget_show_all (cd->combo);
 	return cd->combo;
 }
 
-
-/* Clipboard group.  */
-
-static void
-editor_toolbar_cut_cb (GtkWidget *widget, GtkHTMLControlData *cd)
-{
-	gtk_html_cut (GTK_HTML (cd->html));
-}
-
-static void
-editor_toolbar_copy_cb (GtkWidget *widget, GtkHTMLControlData *cd)
-{
-	gtk_html_copy (GTK_HTML (cd->html));
-}
-
-static void
-editor_toolbar_paste_cb (GtkWidget *widget, GtkHTMLControlData *cd)
-{
-	gtk_html_paste (GTK_HTML (cd->html));
-}
-
 
 /* Font style group.  */
 
@@ -353,18 +300,6 @@ insertion_font_style_changed_cb (GtkHTML *widget, GtkHTMLFontStyle font_style, G
 	cd->block_font_style_change = FALSE;
 }
 
-static void
-color_changed_cb (GtkHTML *widget,
-		  HTMLColor *color,
-		  GtkHTMLControlData *cd)
-{
-	gnome_color_picker_set_d (GNOME_COLOR_PICKER (cd->cpicker),
-				  ((gdouble) color->color.red)   / 0xffff,
-				  ((gdouble) color->color.green) / 0xffff,
-				  ((gdouble) color->color.blue)  / 0xffff,
-				  1.0);
-}
-
 
 /* Alignment group.  */
 
@@ -449,33 +384,16 @@ static void
 editor_toolbar_indent_cb (GtkWidget *widget,
 			  GtkHTMLControlData *cd)
 {
-	gtk_html_indent (GTK_HTML (cd->html), +1);
+	gtk_html_modify_indent_by_delta (GTK_HTML (cd->html), +1);
 }
 
 static void
 editor_toolbar_unindent_cb (GtkWidget *widget,
 			    GtkHTMLControlData *cd)
 {
-	gtk_html_indent (GTK_HTML (cd->html), -1);
+	gtk_html_modify_indent_by_delta (GTK_HTML (cd->html), -1);
 }
 
-/* undo/redo */
-
-static void
-editor_toolbar_undo_cb (GtkWidget *widget,
-			GtkHTMLControlData *cd)
-{
-	gtk_html_undo (GTK_HTML (cd->html));
-}
-
-static void
-editor_toolbar_redo_cb (GtkWidget *widget,
-			GtkHTMLControlData *cd)
-{
-	gtk_html_redo (GTK_HTML (cd->html));
-}
-
-
 
 /* Editor toolbar.  */
 
@@ -489,23 +407,6 @@ static GnomeUIInfo editor_toolbar_alignment_group[] = {
 	GNOMEUIINFO_END
 };
 
-static GnomeUIInfo toolbar_info[] = {
-	GNOMEUIINFO_ITEM_STOCK (N_("Cut"), N_("Cut the selected region to the clipboard"),
-				editor_toolbar_cut_cb, GNOME_STOCK_PIXMAP_CUT),
-	GNOMEUIINFO_ITEM_STOCK (N_("Copy"), N_("Copy the selected region to the clipboard"),
-				editor_toolbar_copy_cb, GNOME_STOCK_PIXMAP_COPY),
-	GNOMEUIINFO_ITEM_STOCK (N_("Paste"), N_("Paste contents of the clipboard"),
-				editor_toolbar_paste_cb, GNOME_STOCK_PIXMAP_PASTE),
-	GNOMEUIINFO_SEPARATOR,
-
-	GNOMEUIINFO_ITEM_STOCK (N_("Undo"), N_("Undo last editor operation"),
-				editor_toolbar_undo_cb, GNOME_STOCK_PIXMAP_UNDO),
-	GNOMEUIINFO_ITEM_STOCK (N_("Redo"), N_("Redo undone editor operation"),
-				editor_toolbar_redo_cb, GNOME_STOCK_PIXMAP_REDO),
-
-	GNOMEUIINFO_END
-};
-
 static GnomeUIInfo editor_toolbar_style_uiinfo[] = {
 
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Bold"), N_("Sets the bold font"),
@@ -599,9 +500,6 @@ create_style_toolbar (GtkHTMLControlData *cd)
 	gtk_signal_connect (GTK_OBJECT (cd->html), "current_paragraph_alignment_changed",
 			    GTK_SIGNAL_FUNC (paragraph_alignment_changed_cb), cd);
 
-	gtk_signal_connect (GTK_OBJECT (cd->html), "insertion_color_changed",
-			    GTK_SIGNAL_FUNC (color_changed_cb), cd);
-
 	return hbox;
 }
 
@@ -614,20 +512,3 @@ toolbar_style (GtkHTMLControlData *cd)
 
 	return create_style_toolbar (cd);
 }
-
-void
-toolbar_setup (BonoboUIComponent  *uic,
-	       GtkHTMLControlData *cd)
-{
-	BonoboUIHandlerToolbarItem *tree;
-	BonoboUIHandler *uih = bonobo_ui_handler_new_from_component (uic);
-
-	g_return_if_fail (uih != NULL);
-	g_return_if_fail (BONOBO_IS_UI_HANDLER (uih));
-	g_return_if_fail (cd->html != NULL);
-	g_return_if_fail (GTK_IS_HTML (cd->html));
-
-	tree = bonobo_ui_handler_toolbar_parse_uiinfo_list_with_data (toolbar_info, cd);
-	bonobo_ui_handler_toolbar_add_list (uih, "/Toolbar", tree);
-	bonobo_ui_handler_toolbar_free_list (tree);
-}
