@@ -22,171 +22,30 @@
 #ifndef _GTKHTML_H_
 #define _GTKHTML_H_
 
-
-typedef struct _GtkHTML	            GtkHTML;
-typedef struct _GtkHTMLPrivate      GtkHTMLPrivate;
-typedef struct _GtkHTMLClass	    GtkHTMLClass;
-typedef struct _GtkHTMLInputLine    GtkHTMLInputLine;
-
 #include <sys/types.h>
-#include <gtk/gtk.h>
+#include <gtk/gtklayout.h>
 #include <libgnomeprint/gnome-print.h>
 
-
+#include "gtkhtml-types.h"
+#include "gtkhtml-enums.h"
+
+/* FIXME we should remove html dep */
+#include "htmltypes.h"
+
 #define GTK_TYPE_HTML                  (gtk_html_get_type ())
 #define GTK_HTML(obj)                  (GTK_CHECK_CAST ((obj), GTK_TYPE_HTML, GtkHTML))
 #define GTK_HTML_CLASS(klass)          (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_HTML, GtkHTMLClass))
 #define GTK_IS_HTML(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_HTML))
 #define GTK_IS_HTML_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_HTML))
 
-#define GTK_HTML_PROPERTY(w,p)         (GTK_HTML_CLASS (GTK_OBJECT (w)->klass)->properties-> ## p)
-
-
-typedef struct _GtkHTMLStream GtkHTMLStream;
-
-enum _GtkHTMLStreamStatus {
-	GTK_HTML_STREAM_OK,
-	GTK_HTML_STREAM_ERROR
-};
-typedef enum _GtkHTMLStreamStatus GtkHTMLStreamStatus;
-
-enum _GtkHTMLParagraphStyle {
-	GTK_HTML_PARAGRAPH_STYLE_NORMAL,
-	GTK_HTML_PARAGRAPH_STYLE_H1,
-	GTK_HTML_PARAGRAPH_STYLE_H2,
-	GTK_HTML_PARAGRAPH_STYLE_H3,
-	GTK_HTML_PARAGRAPH_STYLE_H4,
-	GTK_HTML_PARAGRAPH_STYLE_H5,
-	GTK_HTML_PARAGRAPH_STYLE_H6,
-	GTK_HTML_PARAGRAPH_STYLE_ADDRESS,
-	GTK_HTML_PARAGRAPH_STYLE_PRE,
-	GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED,
-	GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN,
-	GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT
-};
-typedef enum _GtkHTMLParagraphStyle GtkHTMLParagraphStyle;
-
-enum _GtkHTMLParagraphAlignment {
-	GTK_HTML_PARAGRAPH_ALIGNMENT_LEFT,
-	GTK_HTML_PARAGRAPH_ALIGNMENT_RIGHT,
-	GTK_HTML_PARAGRAPH_ALIGNMENT_CENTER
-};
-typedef enum _GtkHTMLParagraphAlignment GtkHTMLParagraphAlignment;
-
-enum _GtkHTMLCursorSkipType {
-	GTK_HTML_CURSOR_SKIP_ONE,
-	GTK_HTML_CURSOR_SKIP_WORD,
-	GTK_HTML_CURSOR_SKIP_PAGE,
-	GTK_HTML_CURSOR_SKIP_ALL
-};
-typedef enum _GtkHTMLCursorSkipType GtkHTMLCursorSkipType;
-
-enum _GtkHTMLCommandType {
-	GTK_HTML_COMMAND_UNDO,
-	GTK_HTML_COMMAND_REDO,
-	GTK_HTML_COMMAND_COPY,
-	GTK_HTML_COMMAND_CUT,
-	GTK_HTML_COMMAND_PASTE,
-
-	GTK_HTML_COMMAND_CUT_LINE,
-
-	GTK_HTML_COMMAND_INSERT_PARAGRAPH,
-	GTK_HTML_COMMAND_INSERT_RULE,
-	GTK_HTML_COMMAND_INSERT_RULE_PARAM,
-	GTK_HTML_COMMAND_INSERT_IMAGE_PARAM,
-
-	GTK_HTML_COMMAND_MAKE_LINK,
-	GTK_HTML_COMMAND_REMOVE_LINK,
-
-	GTK_HTML_COMMAND_DELETE,
-	GTK_HTML_COMMAND_DELETE_BACK,
-
-	GTK_HTML_COMMAND_SET_MARK,
-	GTK_HTML_COMMAND_DISABLE_SELECTION,
-
-	GTK_HTML_COMMAND_TOGGLE_BOLD,
-	GTK_HTML_COMMAND_TOGGLE_ITALIC,
-	GTK_HTML_COMMAND_TOGGLE_UNDERLINE,
-	GTK_HTML_COMMAND_TOGGLE_STRIKEOUT,
-
-	GTK_HTML_COMMAND_SIZE_MINUS_2,
-	GTK_HTML_COMMAND_SIZE_MINUS_1,
-	GTK_HTML_COMMAND_SIZE_PLUS_0,
-	GTK_HTML_COMMAND_SIZE_PLUS_1,
-	GTK_HTML_COMMAND_SIZE_PLUS_2,
-	GTK_HTML_COMMAND_SIZE_PLUS_3,
-	GTK_HTML_COMMAND_SIZE_PLUS_4,
-
-	GTK_HTML_COMMAND_SIZE_INCREASE,
-	GTK_HTML_COMMAND_SIZE_DECREASE,
-
-	GTK_HTML_COMMAND_ALIGN_LEFT,
-	GTK_HTML_COMMAND_ALIGN_CENTER,
-	GTK_HTML_COMMAND_ALIGN_RIGHT,
-
-	GTK_HTML_COMMAND_INDENT_INC,
-	GTK_HTML_COMMAND_INDENT_DEC,
-
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_NORMAL,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H1,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H2,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H3,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H4,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H5,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H6,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ADDRESS,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_PRE,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMDOTTED,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMROMAN,
-	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMDIGIT,
-
-	GTK_HTML_COMMAND_MODIFY_SELECTION_UP,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_DOWN,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_LEFT,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_RIGHT,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_PAGEUP,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_PAGEDOWN,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_BOL,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_EOL,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_BOD,
-	GTK_HTML_COMMAND_MODIFY_SELECTION_EOD,
-
-	GTK_HTML_COMMAND_CAPITALIZE_WORD,
-	GTK_HTML_COMMAND_UPCASE_WORD,
-	GTK_HTML_COMMAND_DOWNCASE_WORD,
-
-	GTK_HTML_COMMAND_SPELL_SUGGEST,
-	GTK_HTML_COMMAND_SPELL_PERSONAL_DICTIONARY_ADD,
-	GTK_HTML_COMMAND_SPELL_SESSION_DICTIONARY_ADD,
-
-	GTK_HTML_COMMAND_SEARCH,
-	GTK_HTML_COMMAND_SEARCH_INCREMENTAL_FORWARD,
-	GTK_HTML_COMMAND_SEARCH_INCREMENTAL_BACKWARD,
-	GTK_HTML_COMMAND_SEARCH_REGEX,
-
-	GTK_HTML_COMMAND_FOCUS_FORWARD,
-	GTK_HTML_COMMAND_FOCUS_BACKWARD
-};
-typedef enum _GtkHTMLCommandType GtkHTMLCommandType;
-
-
-#include "gtkhtml-properties.h"
-#include "gtkhtml-embedded.h"
-
-/* FIXME we should hide this stuff.  */
-#include "htmlengine.h"
-#include "htmlengine-save.h"
-#include "gtkhtmlfontstyle.h"
-
-typedef HTMLEngineSaveReceiverFn GtkHTMLSaveReceiverFn;
-
 struct _GtkHTML {
 	GtkLayout layout;
 
-	GtkBindingSet      *editor_bindings;
-	GtkWidget          *iframe_parent;
-	GtkHTMLInputLine   *input_line;
-	HTMLEngine         *engine;
+	GtkBindingSet        *editor_bindings;
+	GtkWidget            *iframe_parent;
+	GtkHTMLEditorAPI     *editor_api;
+	gpointer              editor_data;
+	HTMLEngine           *engine;
 
 	/* The URL of the link over which the pointer currently is.  NULL if
            the pointer is not over a link.  */
@@ -243,18 +102,37 @@ struct _GtkHTMLClass {
 	void (* scroll)               (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
 	void (* cursor_move)          (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
 	void (* command)              (GtkHTML *html, GtkHTMLCommandType com_type);
-	void (* spell_suggestion_request) (GtkHTML *html, gpointer *spell_checker, gchar *word);
 
 	/* properties */
 	GtkHTMLClassProperties *properties;
 };
 
+struct _GtkHTMLEditorAPI
+{
+	/* spell checking methods */
+	gboolean  (* check_word)              (GtkHTML *html, const gchar *word, gpointer data);
+	void      (* suggestion_request)      (GtkHTML *html, const gchar *word, gpointer data);
+	void      (* add_to_session)          (GtkHTML *html, const gchar *word, gpointer data);
+	void      (* add_to_personal)         (GtkHTML *html, const gchar *word, gpointer data);
+
+	/* unhandled commands */
+	gboolean  (* command)                 (GtkHTML *html, GtkHTMLCommandType com_type, gpointer data);
+
+	GtkArg  * (* event)                   (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **args, gpointer data);
+
+	/* input line */
+	GtkEntry * (* create_input_line)      (GtkHTML *html, gpointer data);
+};
+
 
 
 /* Creation.  */
-GtkType    gtk_html_get_type  (void);
-GtkWidget *gtk_html_new       (void);
-void       gtk_html_construct (GtkWidget *htmlw);
+GtkType    gtk_html_get_type        (void);
+GtkWidget *gtk_html_new             (void);
+void       gtk_html_construct       (GtkWidget        *htmlw);
+void       gtk_html_set_editor_api  (GtkHTML          *html,
+				     GtkHTMLEditorAPI *api,
+				     gpointer          data);
 
 /* parent iframe setting */
 void       gtk_html_set_iframe_parent       (GtkHTML *html,
@@ -270,17 +148,24 @@ void  gtk_html_allow_selection  (GtkHTML  *html,
 void  gtk_html_select_word      (GtkHTML  *html);
 void  gtk_html_select_line      (GtkHTML  *html);
 int   gtk_html_request_paste    (GtkWidget *widget,
+				 gint type,
 				 gint32 time);
 /* Loading.  */
-GtkHTMLStream *gtk_html_begin       (GtkHTML             *html);
-void           gtk_html_write       (GtkHTML             *html,
-				     GtkHTMLStream       *handle,
-				     const gchar         *buffer,
-				     size_t               size);
-void           gtk_html_end         (GtkHTML             *html,
-				     GtkHTMLStream       *handle,
-				     GtkHTMLStreamStatus  status);
-void           gtk_html_load_empty  (GtkHTML             *html);
+GtkHTMLStream *gtk_html_begin             (GtkHTML             *html);
+GtkHTMLStream *gtk_html_begin_content     (GtkHTML             *html,
+					   gchar               *content_type);
+
+void           gtk_html_write             (GtkHTML             *html,
+					   GtkHTMLStream       *handle,
+					   const gchar         *buffer,
+					   size_t               size);
+void           gtk_html_end               (GtkHTML             *html,
+					   GtkHTMLStream       *handle,
+					   GtkHTMLStreamStatus  status);
+void           gtk_html_load_empty        (GtkHTML             *html);
+void           gtk_html_load_from_string  (GtkHTML             *html,
+					   const gchar         *str,
+					   gint                 len);
 
 /* Saving.  */
 gboolean  gtk_html_save  (GtkHTML               *html,
@@ -304,9 +189,6 @@ gboolean  gtk_html_get_editable  (const GtkHTML *html);
 
 /* Printing support.  */
 
-typedef void (*GtkHTMLPrintCallback) (GtkHTML *html, GnomePrintContext *print_context,
-				      gdouble x, gdouble y, gdouble width, gdouble height);
-
 void  gtk_html_print_with_header_footer  (GtkHTML              *html,
 					  GnomePrintContext    *print_context,
 					  gdouble               header_height,
@@ -329,7 +211,9 @@ gboolean  gtk_html_jump_to_anchor  (GtkHTML *html,
 GtkHTMLParagraphStyle	   gtk_html_get_paragraph_style          (GtkHTML                   *html);
 void  			   gtk_html_set_paragraph_style          (GtkHTML                   *html,
 								  GtkHTMLParagraphStyle      style);
-void  			   gtk_html_indent                       (GtkHTML                   *html,
+void  			   gtk_html_set_indent                   (GtkHTML                   *html,
+								  gint                       level);
+void  			   gtk_html_modify_indent_by_delta       (GtkHTML                   *html,
 								  gint                       delta);
 void  			   gtk_html_set_font_style               (GtkHTML                   *html,
 								  GtkHTMLFontStyle           and_mask,
@@ -350,6 +234,9 @@ void  gtk_html_redo  (GtkHTML *html);
 /* misc utils */
 
 void  gtk_html_set_default_background_color (GtkHTML *html, GdkColor *c);
-gpointer gtk_html_get_image_by_imageid (GtkHTML *html, gchar *id);
+void  gtk_html_set_default_content_type (GtkHTML *html, gchar *content_type);
+gpointer  gtk_html_get_object_by_id (GtkHTML *html, const gchar *id);
+
+void  gtk_html_editor_command (GtkHTML *html, const gchar *command_name);
 
 #endif /* _GTKHTML_H_ */
