@@ -24,6 +24,7 @@
 #include <stdarg.h>
 #include <string.h>
 
+#include "gtkhtml.h"
 #include "htmlobject.h"
 #include "htmltext.h"
 #include "htmltextmaster.h"
@@ -32,6 +33,8 @@
 #include "htmlclue.h"
 #include "htmlclueflow.h"
 #include "htmltype.h"
+#include "htmlenums.h"
+#include "htmlenumutils.h"
 
 #include "gtkhtmldebug.h"
 
@@ -92,40 +95,6 @@ clueflow_style_to_string (HTMLClueFlowStyle style)
 	}
 }
 
-static const gchar *
-gtk_html_debug_print_valignment (HTMLVAlignType a)
-{
-	switch (a) {
-	case HTML_VALIGN_TOP:
-		return "Top";
-	case HTML_VALIGN_CENTER:
-		return "Center";
-	case HTML_VALIGN_BOTTOM:
-		return "Bottom";
-	case HTML_VALIGN_NONE:
-		return "None";
-	default:
-		return "Unknown";
-	}
-}
-
-static const gchar *
-gtk_html_debug_print_halignment (HTMLHAlignType a)
-{
-	switch (a) {
-	case HTML_HALIGN_LEFT:
-		return "Left";
-	case HTML_HALIGN_CENTER:
-		return "Center";
-	case HTML_HALIGN_RIGHT:
-		return "Right";
-	case HTML_HALIGN_NONE:
-		return "None";
-	default:
-		return "Unknown";
-	}
-}
-
 
 void
 gtk_html_debug_dump_table (HTMLObject *o,
@@ -150,9 +119,9 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 	for (i = 0; i < level; i++)
 		g_print (" ");
 
-	g_print ("Obj: %p, Parent: %p  Prev: %p Next: %p ObjectType: %s Pos: %d, %d, MinWidth: %d, Width: %d MaxWidth: %d ",
-		 obj, obj->parent, obj->prev, obj->next, html_type_name (HTML_OBJECT_TYPE (obj)),
-		 obj->x, obj->y, obj->min_width, obj->width, obj->max_width);
+	g_print ("ObjectType: %s Pos: %d, %d, MinWidth: %d, Width: %d MaxWidth: %d Ascent %d Descent %d",
+		 html_type_name (HTML_OBJECT_TYPE (obj)),
+		 obj->x, obj->y, obj->min_width, obj->width, obj->max_width, obj->ascent, obj->descent);
 
 	if (HTML_OBJECT_TYPE (obj) == HTML_TYPE_CLUEFLOW)
 		g_print (" [%s, %d]",
@@ -196,8 +165,8 @@ gtk_html_debug_dump_object (HTMLObject *obj,
 	case HTML_TYPE_TABLECELL:
 		for (i = 0; i < level; i++) g_print (" ");
 		g_print ("HAlign: %s VAlign: %s\n",
-			 gtk_html_debug_print_halignment (HTML_CLUE (obj)->halign),
-			 gtk_html_debug_print_valignment (HTML_CLUE (obj)->valign));
+			 html_halign_name (HTML_CLUE (obj)->halign),
+			 html_valign_name (HTML_CLUE (obj)->valign));
 		gtk_html_debug_dump_tree (HTML_CLUE (obj)->head, level + 1);
 		break;
 
