@@ -20,9 +20,11 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 */
+
+#include <config.h>
 #include "htmlclue.h"
-#include "htmlcluev.h"
 #include "htmlsearch.h"
+#include "htmltype.h"
 
 
 #define HC_CLASS(x) (HTML_CLUE_CLASS (HTML_OBJECT (x)->klass))
@@ -247,8 +249,8 @@ check_point (HTMLObject *o,
 	HTMLObject *obj;
 	HTMLObject *obj2;
 
-	if (x < o->x || x > o->x + o->width
-	    || y > o->y + o->descent || y < o->y - o->ascent)
+	if (x < o->x || x >= o->x + o->width
+	    || y >= o->y + o->descent || y < o->y - o->ascent)
 		return NULL;
 
 	x = x - o->x;
@@ -274,9 +276,6 @@ check_page_split (HTMLObject *self,
 	gint last_under = 0;
 
 	clue = HTML_CLUE (self);
-
-	y -= (self->y - self->ascent);
-
 	for (p = clue->head; p != NULL; p = p->next) {
 		gint y1, y2;
 
@@ -296,15 +295,16 @@ check_page_split (HTMLObject *self,
 
 static void
 forall (HTMLObject *self,
+	HTMLEngine *e,
 	HTMLObjectForallFunc func,
 	gpointer data)
 {
 	HTMLObject *p;
 
 	for (p = HTML_CLUE (self)->head; p != NULL; p = p->next)
-		html_object_forall (p, func, data);
+		html_object_forall (p, e, func, data);
 
-	html_object_class.forall (self, func, data);
+	html_object_class.forall (self, e, func, data);
 }
 
 static gboolean
@@ -429,26 +429,6 @@ append_selection_string (HTMLObject *self,
 	}
 }
 
-static HTMLObject *
-next (HTMLObject *self, HTMLObject *child)
-{
-	g_assert (self);
-	g_assert (child);
-	g_assert (child->parent == self);
-
-	return html_object_next_not_slave (child);
-}
-
-static HTMLObject *
-prev (HTMLObject *self, HTMLObject *child)
-{
-	g_assert (self);
-	g_assert (child);
-	g_assert (child->parent == self);
-
-	return html_object_prev_not_slave (child);
-}
-
 static HTMLObject *
 head (HTMLObject *self)
 {
@@ -502,8 +482,6 @@ html_clue_class_init (HTMLClueClass *klass,
 	object_class->save_plain = save_plain;
 	object_class->search = search;
 	object_class->append_selection_string = append_selection_string;
-	object_class->next = next;
-	object_class->prev = prev;
 	object_class->head = head;
 	object_class->tail = tail;
 
