@@ -28,9 +28,13 @@
 #include <config.h>
 #include <ctype.h>
 #include <string.h>
+#include "gtkhtml-properties.h"
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
 #include "htmlclue.h"
 #include "htmlclueflow.h"
 #include "htmlcluealigned.h"
+#include "htmlpainter.h"
 #include "htmltext.h"
 #include "htmlvspace.h"
 #include "htmllinktextmaster.h"
@@ -38,6 +42,7 @@
 #include "htmlsearch.h"
 #include "htmlentity.h"
 #include "htmlengine-edit.h"
+#include "htmlengine-save.h"
 
 
 HTMLClueFlowClass html_clueflow_class;
@@ -731,14 +736,15 @@ draw (HTMLObject *self,
 
 		bullet_size = calc_bullet_size (painter);
 
-		/* FIXME pen color?  */
-		
 		xp = self->x + first->x - 2 * bullet_size;
-		yp = self->y - self->ascent + first->y - (first->ascent + bullet_size) / 2 - bullet_size;
+		yp = self->y - self->ascent 
+			+ (first->y - first->ascent) 
+			+ (first->ascent + first->descent)/2 
+			- bullet_size/2;
 
 		xp += tx, yp += ty;
 
-		html_painter_set_pen (painter,
+	 	html_painter_set_pen (painter,
 				      &html_colorset_get_color_allocated (painter, HTMLTextColor)->color);
 
 		if (clueflow->level == 0 || (clueflow->level & 1) != 0)
@@ -785,9 +791,7 @@ check_point (HTMLObject *self,
 	     guint *offset_return,
 	     gboolean for_cursor)
 {
-	HTMLObject *obj;
-	HTMLObject *p;
-	HTMLObject *pnext;
+	HTMLObject *obj, *p, *pnext;
 	HTMLClue *clue;
 
 	if (x < self->x || x >= self->x + self->width
@@ -800,8 +804,6 @@ check_point (HTMLObject *self,
 	y = y - self->y + self->ascent;
 
 	for (p = clue->head; p != NULL; p = pnext) {
-		gint x1, y1;
-
 		pnext = p->next;
 
 		obj = html_object_check_point (p, painter, x, y, offset_return, for_cursor);
@@ -811,30 +813,17 @@ check_point (HTMLObject *self,
 		if (! for_cursor)
 			continue;
 
-		if (p->prev == NULL && (x < self->x || y < self->y - self->ascent + 1)) {
-			x1 = self->x;
-			y1 = self->y - self->ascent + 1;
-			obj = html_object_check_point (p, painter, x1, y1, offset_return, for_cursor);
+		if (p->prev == NULL && (x < p->x || y < p->y - p->ascent)) {
+			obj = html_object_check_point (p, painter, p->x, p->y - p->ascent, offset_return, for_cursor);
 			if (obj != NULL)
 				return obj;
 		} else if (pnext == NULL || (pnext->y != p->y
-					     && y >= p->y - p->ascent + 1
-					     && y <= p->y + p->descent)) {
-			HTMLObject *obj1;
-
-			x1 = p->x + p->width - 1;
-			y1 = p->y;
-
-			if (HTML_OBJECT_TYPE (p) != HTML_TYPE_TEXTSLAVE) {
-				obj1 = p;
-			} else {
-				for (obj1 = p;
-				     HTML_OBJECT_TYPE (obj1) == HTML_TYPE_TEXTSLAVE;
-				     obj1 = obj1->prev)
-					;
-			}
-
-			obj = html_object_check_point (obj1, painter, x1, y1, offset_return, for_cursor);
+					     && y >= p->y - p->ascent
+					     && y <= p->y + p->descent - 1)) {
+			obj = html_object_check_point (HTML_OBJECT_TYPE (p) != HTML_TYPE_TEXTSLAVE
+						       ? p : html_object_prev_not_slave (p),
+						       painter, p->x + p->width - 1, p->y + p->descent -1,
+						       offset_return, for_cursor);
 			if (obj != NULL)
 				return obj;
 		}
@@ -974,53 +963,6 @@ write_post_tags (HTMLClueFlow *self,
 	return TRUE;
 }
 
-static const gchar *
-get_tag_for_style (const HTMLClueFlow *flow)
-{
-	switch (flow->style) {
-	case HTML_CLUEFLOW_STYLE_NORMAL:
-		return NULL;
-	case HTML_CLUEFLOW_STYLE_H1:
-		return "h1";
-	case HTML_CLUEFLOW_STYLE_H2:
-		return "h2";
-	case HTML_CLUEFLOW_STYLE_H3:
-		return "h3";
-	case HTML_CLUEFLOW_STYLE_H4:
-		return "h4";
-	case HTML_CLUEFLOW_STYLE_H5:
-		return "h5";
-	case HTML_CLUEFLOW_STYLE_H6:
-		return "h6";
-	case HTML_CLUEFLOW_STYLE_ADDRESS:
-		return "address";
-	case HTML_CLUEFLOW_STYLE_PRE:	
-		return "pre";
-	case HTML_CLUEFLOW_STYLE_ITEMDOTTED:
-	case HTML_CLUEFLOW_STYLE_ITEMROMAN:
-	case HTML_CLUEFLOW_STYLE_ITEMDIGIT:
-		return "li";
-	default:
-		g_warning ("Unknown HTMLClueFlowStyle %d", flow->style);
-		return NULL;
-	}
-}
-
-static const gchar *
-halign_to_string (HTMLHAlignType halign)
-{
-	switch (halign) {
-	case HTML_HALIGN_RIGHT:
-		return "right";
-	case HTML_HALIGN_CENTER:
-		return "center";
-	case HTML_HALIGN_LEFT:
-	case HTML_HALIGN_NONE:
-	default:
-		return "left";
-	}
-}
-
 static gboolean 
 is_similar (HTMLObject *self, HTMLObject *friend)
 {
@@ -1049,7 +991,7 @@ save (HTMLObject *self,
 	if (! write_pre_tags (clueflow, state))
 		return FALSE;
 
-	tag = get_tag_for_style (clueflow);
+	tag = html_engine_save_get_paragraph_style (clueflow_style_to_paragraph_style (clueflow->style));
 
 	if (is_similar (self, self->prev))
 		start = FALSE;
@@ -1067,7 +1009,9 @@ save (HTMLObject *self,
 	
 	/* Alignment tag.  */
 	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
-		if (! html_engine_save_output_string (state, "<div alignx=%s>\n", halign_to_string (halign)))
+		if (! html_engine_save_output_string
+		    (state, "<div align=%s>\n",
+		     html_engine_save_get_paragraph_align (html_alignment_to_paragraph (halign))))
 			return FALSE;
 	}
 
@@ -1099,16 +1043,22 @@ save (HTMLObject *self,
 	return write_post_tags (HTML_CLUEFLOW (self), state);
 }
 
+#define MATCH_UTF8_NBSP(s) (*s == (guchar)0xc2 && *(s + 1) == (guchar)0xa0)
+ 
 static gint
-string_append_len (GString *out, guchar *s, gint length)
+string_append_nonbsp (GString *out, guchar *s, gint length)
 {
 	gint len = length;
-	guchar c;
 
 	while (len--) {
-		c = (*s == ENTITY_NBSP) ? ' ' : *s;
-		g_string_append_c (out, c);
-		s++;
+		if (MATCH_UTF8_NBSP (s)) {
+			g_string_append_c (out, ' ');
+			s += 2;
+			len--;
+		} else {
+			g_string_append_c (out, *s);
+			s++;
+		}
 	}
 	return length;
 }
@@ -1177,37 +1127,29 @@ save_plain (HTMLObject *self,
 			/* FIXME we should allow wrapping on PRE sections as well */
 			len = strcspn (s, "\n");
 			
-			switch (flow->style) {
-			case HTML_CLUEFLOW_STYLE_PRE:
-			case HTML_CLUEFLOW_STYLE_NOWRAP:
-
-				plain_padding (flow, out, firstline);
-				s += string_append_len (out, s, len);
-
-				break;
-			default:
-
-				if (len > (requested_width - pad)) {
-					space = s + (requested_width - pad);
-					while (space > s && (*space != ' '
-							     || (*(space + 1) == ENTITY_NBSP)
-							     || (*(space - 1) == ENTITY_NBSP)))
-						/* space = unicode_previous_utf8 (s, space); */
-						space = space - 1;
+			if (flow->style != HTML_CLUEFLOW_STYLE_PRE
+			    && flow->style != HTML_CLUEFLOW_STYLE_NOWRAP) {
+				
+				if (unicode_strlen (s, len) > (requested_width - pad)) {
+					space = s + unicode_offset_to_index (s, requested_width - pad);
+					while (space > s 
+					       && (*space != ' '))
+						// || (MATCH_UTF8_NBSP ((guchar *)unicode_next_utf8 (space)))
+						// || (MATCH_UTF8_NBSP ((guchar *)unicode_previous_utf8 (s, space)))))
+						space = unicode_previous_utf8 (s, space);
 					
 					if (space != s)
 						len = space - s;
 				}
-
-				plain_padding (flow, out, firstline);
-				s += string_append_len (out, s, len);
-
-				while (*s == ' ' || *s == ENTITY_NBSP) 
-					s++;
-				
-				break;
-			}							
-
+			}
+			
+		        plain_padding (flow, out, firstline);
+			s += string_append_nonbsp (out, s, len);
+			
+			/* Trim the space at the end */
+			while (*s == ' ' || MATCH_UTF8_NBSP (s)) 
+				s = unicode_next_utf8 (s);
+			
 			if (*s == '\n') 
 				s++;
 			
@@ -1221,7 +1163,7 @@ save_plain (HTMLObject *self,
 	}
 	html_engine_save_buffer_free (buffer_state);
 		
-	if (!html_engine_save_output_string (state, out->str)) {
+	if (!html_engine_save_output_string (state, "%s", out->str)) {
 		g_string_free (out, TRUE);
 		return FALSE;
 	}		
@@ -1544,7 +1486,6 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->draw_background = draw_background;
 	object_class->save = save;
 	object_class->save_plain = save_plain;
-	// object_class->check_page_split = check_page_split;
 	object_class->check_point = check_point;
 	object_class->append_selection_string = append_selection_string;
 	object_class->search = search;
@@ -1591,6 +1532,17 @@ html_clueflow_new (HTMLClueFlowStyle style,
 	return HTML_OBJECT (clueflow);
 }
 
+HTMLObject *
+html_clueflow_new_from_flow (HTMLClueFlow *flow)
+{
+	HTMLObject *o;
+
+	o = html_clueflow_new (flow->style, flow->level);
+	html_object_copy_data_from_object (o, HTML_OBJECT (flow));
+
+	return o;
+}
+
 
 /* Virtual methods.  */
 
@@ -1627,7 +1579,7 @@ html_clueflow_split (HTMLClueFlow *clue,
 
 	/* Create the new clue.  */
 
-	new = HTML_CLUEFLOW (html_clueflow_new (clue->style, clue->level));
+	new = HTML_CLUEFLOW (html_clueflow_new_from_flow (clue));
 
 	/* Remove the children from the original clue.  */
 
@@ -1734,21 +1686,21 @@ html_clueflow_get_halignment (HTMLClueFlow *flow)
 }
 
 void
-html_clueflow_indent (HTMLClueFlow *flow,
-		      HTMLEngine *engine,
-		      gint indentation)
+html_clueflow_modify_indentation_by_delta (HTMLClueFlow *flow,
+					   HTMLEngine *engine,
+					   gint indentation_delta)
 {
 	g_return_if_fail (flow != NULL);
 	g_return_if_fail (engine != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (engine));
 
-	if (indentation == 0)
+	if (indentation_delta == 0)
 		return;
 
-	if (indentation > 0) {
-		flow->level += indentation;
-	} else if ((- indentation) < flow->level) {
-		flow->level += indentation;
+	if (indentation_delta > 0) {
+		flow->level += indentation_delta;
+	} else if ((- indentation_delta) < flow->level) {
+		flow->level += indentation_delta;
 	} else if (flow->level != 0) {
 		flow->level = 0;
 	} else {
@@ -1840,8 +1792,6 @@ html_clueflow_remove_text_slaves (HTMLClueFlow *flow)
 	}
 }
 
-#ifdef GTKHTML_HAVE_PSPELL
-
 /* spell checking */
 
 #include "htmlinterval.h"
@@ -1853,14 +1803,14 @@ get_text_bytes (HTMLClue *clue, HTMLInterval *i)
 	guint bytes;
 
 	g_assert (i);
-	g_assert (i->from);
-	g_assert (i->to);
+	g_assert (i->from.object);
+	g_assert (i->to.object);
 
 	bytes = 0;
-	obj = i->from;
+	obj = i->from.object;
 	while (1) {
 		bytes += html_interval_get_bytes (i, obj);
-		if (obj == i->to) break;
+		if (obj == i->to.object) break;
 		obj = obj->next;
 		g_assert (obj);
 	}
@@ -1879,7 +1829,7 @@ get_text (HTMLClue *clue, HTMLInterval *i)
 	ct         = text = g_malloc (bytes + 1);
 	text [bytes] = 0;
 
-	obj = i->from;
+	obj = i->from.object;
 	while (obj) {
 		cb = html_interval_get_bytes (i, obj);
 		if (html_object_is_text (obj))
@@ -1888,7 +1838,7 @@ get_text (HTMLClue *clue, HTMLInterval *i)
 			if (cb == 1) *ct = ' ';
 			else memset (ct, ' ', cb);
 		ct += cb;
-		if (obj == i->to) break;
+		if (obj == i->to.object) break;
 		obj = obj->next;
 	}
 
@@ -1991,15 +1941,16 @@ html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e, HTMLInterval *i)
 	   printf ("html_clueflow_spell_check %p %p %d %d\n", i->from, i->to, i->from_offset, i->to_offset); */
 
 	clue = HTML_CLUE (flow);
-	if (!GTK_HTML_CLASS (GTK_OBJECT (e->widget)->klass)->properties->live_spell_check
-	    || !e->spell_checker || !clue || !clue->tail)
+	if (!e->widget->editor_api
+	    || !GTK_HTML_CLASS (GTK_OBJECT (e->widget)->klass)->properties->live_spell_check
+	    || !clue || !clue->tail)
 		return;
 
 	off  = 0;
 	if (!i)
 		i = html_interval_new (clue->head, clue->tail, 0, html_object_get_length (clue->tail));
 	text = get_text (clue, i);
-	obj  = i->from;
+	obj  = i->from.object;
 	if (obj && html_object_is_text (obj))
 		html_text_spell_errors_clear_interval (HTML_TEXT (obj), i);
 
@@ -2017,7 +1968,7 @@ html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e, HTMLInterval *i)
 				bak = *ct;
 				*ct = 0;
 				/* printf ("off %d going to test word: \"%s\"\n", off, word); */
-				result = pspell_manager_check (e->spell_checker, word);
+				result = (*e->widget->editor_api->check_word) (e->widget, word, e->widget->editor_data);
 
 				if (result == 1) {
 					gboolean is_text = (obj) ? html_object_is_text (obj) : FALSE;
@@ -2025,16 +1976,8 @@ html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e, HTMLInterval *i)
 						       || (is_text && off + html_interval_get_length (i, obj)
 							   < unicode_index_to_offset (text, ct - text))))
 						obj = next_obj_and_clear (obj, &off, &is_text, i);
-				} else {
-					if (result == 0) {
-						if (obj)
-							obj = spell_check_word_mark (obj, text, word, &off, i);
-					} else if (result == -1)
-						g_warning ("pspell error: %s\n",
-							   pspell_manager_error_message (e->spell_checker));
-					else
-						g_assert_not_reached ();
-				}
+				} else if (obj)
+						obj = spell_check_word_mark (obj, text, word, &off, i);
 
 				*ct = bak;
 				if (*ct)
@@ -2043,5 +1986,3 @@ html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e, HTMLInterval *i)
 		}
 	}
 }
-
-#endif /* GTKHTML_HAVE_PSPELL */
