@@ -24,19 +24,19 @@
    movement.  But for now, I think this is a reasonable way to get things to
    work.  */
 
+#include <config.h>
 #include <glib.h>
 
 #include "htmlclue.h"
+#include "htmlengine.h"
 #include "htmltext.h"
 #include "htmltextslave.h"
 #include "htmltype.h"
 
 #include "htmlcursor.h"
 
-
-#define _HTML_CURSOR_DEBUG
+/* #define _HTML_CURSOR_DEBUG */
 
-
 #ifdef _HTML_CURSOR_DEBUG
 static void
 debug_location (const HTMLCursor *cursor)
@@ -63,17 +63,8 @@ normalize (HTMLObject **object,
 	   guint *offset)
 {
 	if (*offset == 0 && (*object)->prev != NULL) {
-		HTMLObject *p;
-
-		p = (*object)->prev;
-		while (p != NULL && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE)
-			p = p->prev;
-
-		*object = p;
-		if (html_object_is_text (p))
-			*offset = HTML_TEXT (p)->text_len;
-		else
-			*offset = 1;
+		*object = html_object_prev_not_slave (*object);
+		*offset = html_object_get_length (*object);
 	}
 }
 
@@ -198,7 +189,7 @@ forward (HTMLCursor *cursor)
 	if (!html_object_cursor_forward (cursor->object, cursor)) {
 		HTMLObject *next;
 
-		next = html_object_next_leaf (cursor->object);
+		next = html_object_next_leaf_not_type (cursor->object, HTML_TYPE_TEXTSLAVE);
 		if (next) {
 			cursor->offset = (next->parent == cursor->object->parent) ? 1 : 0;
 			cursor->object = next;
@@ -236,7 +227,7 @@ backward (HTMLCursor *cursor)
 	if (!html_object_cursor_backward (cursor->object, cursor)) {
 		HTMLObject *prev;
 
-		prev = html_object_prev_leaf (cursor->object);
+		prev = html_object_prev_leaf_not_type (cursor->object, HTML_TYPE_TEXTSLAVE);
 		if (prev) {
 			cursor->object = prev;
 			cursor->offset = html_object_get_length (prev);
