@@ -19,9 +19,13 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include "htmlengine-edit-cursor.h"
-
 #include "htmldrawqueue.h"
+#include "htmlengine.h"
+#include "htmlpainter.h"
+#include "htmlobject.h"
+#include "gtkhtml.h"
 
 
 /* HTMLDrawQueueClearElement handling.  */
@@ -227,7 +231,7 @@ draw_obj (HTMLDrawQueue *queue,
 	tx = 0;
 	ty = 0;
 
-	for (p = obj->parent; p != NULL; p = p->parent) {
+	for (p = obj->parent; p != NULL && HTML_OBJECT_TYPE (p) != HTML_TYPE_IFRAME; p = p->parent) {
 		tx += p->x;
 		ty += p->y - p->ascent;
 	}
