@@ -20,9 +20,15 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 */
+
+#include <config.h>
 #include <string.h>
 #include <stdio.h>
+#include "gtkhtml.h"
+#include "gtkhtml-embedded.h"
 #include "htmlembedded.h"
+#include "htmlpainter.h"
+#include "htmlengine.h"
 
 
 HTMLEmbeddedClass html_embedded_class;
@@ -346,7 +352,7 @@ allocate (GtkWidget *w, GtkAllocation  *allocation, HTMLEmbedded *e)
 {
 	if (e->width != allocation->width || e->height != allocation->height) {
 		if (e->width != allocation->width) {
-			html_object_change_set (HTML_OBJECT (e), HTML_CHANGE_MIN_WIDTH);
+			html_object_change_set (HTML_OBJECT (e), HTML_CHANGE_ALL);
 			e->width = allocation->width;
 		}
 		e->height = allocation->height;
