@@ -19,7 +19,12 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include "htmlengine-edit-clueflowstyle.h"
+#include "htmlundo.h"
+#include "htmltype.h"
+#include "htmlcursor.h"
+#include "htmlselection.h"
 
 
 /* Properties of a paragraph.  */
@@ -82,7 +87,7 @@ get_props_and_set (HTMLEngine *engine,
 		   HTMLClueFlow *clueflow,
 		   HTMLClueFlowStyle style,
 		   HTMLHAlignType alignment,
-		   gint indentation_delta,
+		   gint indentation,
 		   HTMLEngineSetClueFlowStyleMask mask)
 {
 	ClueFlowProps *props;
@@ -95,7 +100,10 @@ get_props_and_set (HTMLEngine *engine,
 		html_clueflow_set_halignment (clueflow, engine, alignment);
 
 	if (mask & HTML_ENGINE_SET_CLUEFLOW_INDENTATION)
-		html_clueflow_indent (clueflow, engine, indentation_delta);
+		html_clueflow_set_indentation (clueflow, engine, indentation);
+
+	if (mask & HTML_ENGINE_SET_CLUEFLOW_INDENTATION_DELTA)
+		html_clueflow_modify_indentation_by_delta (clueflow, engine, indentation);
 
 	return props;
 }
@@ -349,7 +357,7 @@ html_engine_set_clueflow_style (HTMLEngine *engine,
 
 	html_undo_discard_redo (engine->undo);
 
-	if (engine->active_selection)
+	if (html_engine_is_selection_active (engine))
 		set_clueflow_style_in_region (engine,
 					      style, alignment, indentation_delta,
 					      mask,
