@@ -19,7 +19,11 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include "htmlcursor.h"
+#include "htmlengine.h"
 #include "htmlengine-edit-cursor.h"
+#include "htmlobject.h"
 
 
 #define BLINK_TIMEOUT 500
