@@ -19,16 +19,21 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include "htmlclue.h"
 #include "htmlclueflow.h"
 #include "htmltext.h"
 #include "htmltextmaster.h"
+#include "htmlselection.h"
+#include "htmlcursor.h"
+#include "htmlundo.h"
 
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-cursor.h"
 #include "htmlengine-edit-delete.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-paste.h"
+#include "htmlengine-edit-selection-updater.h"
 #include "htmlengine-cutbuffer.h"
 
 
@@ -638,8 +643,6 @@ html_engine_delete (HTMLEngine *e,
 	e->cursor->position = start_position;
 
 	html_cursor_normalize (e->cursor);
-	html_engine_show_cursor (e);
-
 	if (do_undo) {
 		/* FIXME this is a nasty workaround for the common case.  The above code
                    to set up the save buffer is utterly broken and we should use the code
@@ -651,17 +654,16 @@ html_engine_delete (HTMLEngine *e,
 			html_engine_cut_buffer_destroy (save_buffer);
 
 			clueflow_orig = HTML_CLUEFLOW (start_parent);
-			clueflow_copy = html_clueflow_new (clueflow_orig->style,
-							   clueflow_orig->level);
+			clueflow_copy = html_clueflow_new_from_flow (clueflow_orig);
 
 			save_buffer = g_list_prepend (NULL, clueflow_copy);
 		}
 
 		setup_undo (e, create_action_data (save_buffer, backwards));
 	}
-#ifdef GTKHTML_HAVE_PSPELL
 	html_engine_spell_check_range (e, e->cursor, e->cursor);
-#endif
+	html_engine_schedule_update (e);
+	html_engine_show_cursor (e);
 }
 
 
