@@ -22,8 +22,8 @@
 #ifndef _HTMLENGINE_EDIT_IMAGES_H
 #define _HTMLENGINE_EDIT_IMAGES_H
 
-#include "htmlengine.h"
-#include "htmlobject.h"
+#include "htmltypes.h"
+#include "htmlenums.h"
 
 void  html_engine_insert_image  (HTMLEngine     *e,
 				 const gchar    *file,
@@ -31,7 +31,8 @@ void  html_engine_insert_image  (HTMLEngine     *e,
 				 const gchar    *target,
 				 gint16          width,
 				 gint16          height,
-				 gint8           percent,
+				 gboolean        percent_width,
+				 gboolean        percent_height,
 				 gint8           border,
 				 HTMLColor *border_color,
 				 HTMLHAlignType  halign,
