@@ -19,13 +19,24 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include <string.h>
 
+#include "gtkhtml-private.h"
+#include "gtkhtml-properties.h"
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
 #include "htmlclue.h"
 #include "htmlclueflow.h"
+#include "htmlcursor.h"
+#include "htmlengine.h"
 #include "htmltext.h"
 #include "htmltextmaster.h"
 #include "htmllinktextmaster.h"
+#include "htmlselection.h"
+#include "htmlsettings.h"
+#include "htmltype.h"
+#include "htmlundo.h"
 
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-cursor.h"
@@ -98,8 +109,7 @@ insert_para (HTMLEngine *engine)
 								      current->next));
 		} else {
 			/* FIXME we need a `html_clueflow_like_another_one()'.  */
-			next_flow = html_clueflow_new (HTML_CLUEFLOW (flow)->style,
-						       HTML_CLUEFLOW (flow)->level);
+			next_flow = html_clueflow_new_from_flow (HTML_CLUEFLOW (flow));
 		}
 	} else {
 		next_flow = HTML_OBJECT (html_clueflow_split (HTML_CLUEFLOW (flow), current));
@@ -145,12 +155,13 @@ insert_para (HTMLEngine *engine)
 
 	engine->cursor->position++;
 
-#ifdef GTKHTML_HAVE_PSPELL
 	html_engine_spell_check_range (engine, orig, orig);
 	html_engine_spell_check_range (engine, engine->cursor, engine->cursor);
-#endif
+
 	html_cursor_destroy (orig);
 	html_engine_show_cursor (engine);
+
+	gtk_html_editor_event_command (engine->widget, GTK_HTML_COMMAND_INSERT_PARAGRAPH);
 }
 
 
@@ -335,9 +346,7 @@ insert_chars (HTMLEngine *e,
 	}
 
 	html_engine_move_cursor (e, HTML_ENGINE_CURSOR_RIGHT, rv);
-#ifdef GTKHTML_HAVE_PSPELL
 	html_engine_spell_check_range (e, orig, e->cursor);
-#endif
 	html_cursor_destroy (orig);
 
 	return rv;
@@ -575,7 +584,7 @@ html_engine_insert (HTMLEngine *e,
 
 	/* magic links */
 	if (GTK_HTML_PROPERTY (e->widget, magic_links) && len == 1
-	    && (text [0] == ' ' || text [0] == '\n')
+	    && (text [0] == ' ' || text [0] == '\n' || text [0] == '>' || text [0] == ')')
 	    && HTML_OBJECT_TYPE (current_object) == HTML_TYPE_TEXTMASTER)
 		html_text_master_magic_link (HTML_TEXT_MASTER (current_object),
 					     e, current_offset);
@@ -591,7 +600,7 @@ html_engine_insert_link (HTMLEngine *e, const gchar *url, const gchar *target)
 	HTMLObject *linked;
 	HTMLColor *color = html_colorset_get_color (e->settings->color_set, HTMLLinkColor);
 
-	if (e->active_selection) {
+	if (html_engine_is_selection_active (e)) {
 		GList *cur;
 
 		html_engine_cut_and_paste_begin (e, "Insert link");
@@ -618,7 +627,7 @@ html_engine_remove_link (HTMLEngine *e)
 	HTMLColor *color = html_colorset_get_color (e->settings->color_set, HTMLTextColor);
 	GList *cur;
 
-	if (e->active_selection) {
+	if (html_engine_is_selection_active (e)) {
 		html_engine_cut_and_paste_begin (e, "Remove link");
 		cur = e->cut_buffer;
 		while (cur) {
