@@ -19,10 +19,16 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include "htmlcolorset.h"
 #include "htmlclue.h"
 #include "htmlclueflow.h"
+#include "htmlcursor.h"
 #include "htmltext.h"
 #include "htmltextmaster.h"
+#include "htmlselection.h"
+#include "htmlsettings.h"
+#include "htmlundo.h"
 
 #include "htmlengine-cutbuffer.h"
 #include "htmlengine-edit.h"
@@ -30,6 +36,7 @@
 #include "htmlengine-edit-cut.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-delete.h"
+#include "htmlengine-edit-selection-updater.h"
 
 #include "gtkhtmldebug.h"
 
@@ -221,13 +228,19 @@ prepare_clueflows (HTMLEngine *engine,
 			first = FALSE;
 			retval = TRUE;
 		} else {	/* ! first || append */
-			if (first) { /* first && append */
+			if (first && engine->cursor->object->next) { /* first && append */
 				split_first_clueflow_at_cursor (engine, HTML_CLUEFLOW (obj), engine->cursor->object->next);
 				clue = engine->cursor->object->parent;
 				first = FALSE;
 			} else {
+				if (!engine->cursor->object->next && first) {
+					clue = engine->cursor->object->parent;
+				}
 				g_assert (clue != NULL);
 				clue = add_new_clueflow (engine, HTML_CLUEFLOW (obj), clue);
+				if (!engine->cursor->object->next) {
+					first = FALSE;
+				}
 			}
 		}
 	}
@@ -744,7 +757,7 @@ html_engine_paste (HTMLEngine *engine,
 	orig_indentation = html_clueflow_get_indentation (current_clueflow);
 
 	/* Cut current selection.  */
-	if (engine->active_selection) {
+	if (html_engine_is_selection_active (engine)) {
 		GList *cut_buffer;
 
 		/* Keep cut buffer.  */
