@@ -19,16 +19,19 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include <ctype.h>
+#include "htmlcursor.h"
 #include "htmlengine-edit-text.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit.h"
 #include "htmltext.h"
+#include "htmlselection.h"
 
 static gboolean
 find_first (HTMLEngine *e)
 {
-	gchar c;
+	guchar c;
 
 	c = html_cursor_get_current_char (e->cursor);
 	while (c == 0 || ! isalnum ((gint) c) || c == ' ') {
@@ -44,7 +47,7 @@ static void
 upper_lower (HTMLObject *obj, gpointer data)
 {
 	gboolean up = GPOINTER_TO_INT (data);
-	gchar *text;
+	guchar *text;
 
 	g_assert (html_object_is_text (obj));
 
