@@ -22,6 +22,7 @@
 */
 
 
+#include <config.h>
 #include <ctype.h>
 #include <glib.h>
 
@@ -31,12 +32,16 @@
 #include "htmltext.h"
 #include "htmltextslave.h"
 #include "htmlimage.h"
+#include "htmlinterval.h"
+#include "htmlselection.h"
+#include "htmlundo.h"
 
 #include "htmlengine-cutbuffer.h"
 #include "htmlengine-edit-cut.h"
 #include "htmlengine-edit-cursor.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-paste.h"
+#include "htmlengine-edit-selection-updater.h"
 #include "htmlengine-edit.h"
 
 
@@ -83,7 +88,6 @@ html_engine_set_mark (HTMLEngine *e)
 		html_engine_unselect_all (e);
 
 	e->mark = html_cursor_dup (e->cursor);
-	e->active_selection = TRUE;
 
 	html_engine_edit_selection_updater_reset (e->selection_updater);
 	html_engine_edit_selection_updater_schedule (e->selection_updater);
@@ -108,7 +112,7 @@ html_engine_cut_buffer_pop (HTMLEngine *e)
 void
 html_engine_selection_push (HTMLEngine *e)
 {
-	if (e->active_selection) {
+	if (html_engine_is_selection_active (e)) {
 		e->selection_stack
 			= g_list_prepend (e->selection_stack, GINT_TO_POINTER (html_cursor_get_position (e->mark)));
 		e->selection_stack
@@ -177,7 +181,6 @@ html_engine_cut_and_paste (HTMLEngine *e, gchar *op_name, GFunc iterator, gpoint
 	html_engine_cut_and_paste_end (e);
 }
 
-#ifdef GTKHTML_HAVE_PSPELL
 void
 html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end)
 {
@@ -202,13 +205,12 @@ html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end
 	html_cursor_destroy (begin);
 	html_cursor_destroy (end);
 }
-#endif
 
 gboolean
 html_is_in_word (unicode_char_t uc)
 {
 	/* printf ("test %d %c => %d\n", uc, uc, unicode_isalnum (uc) || uc == '\''); */
-	return unicode_isalnum (uc) || uc == '\'';
+	return unicode_isalpha (uc) || uc == '\'';
 }
 
 void
@@ -228,3 +230,36 @@ html_engine_select_line_editable (HTMLEngine *e)
 	html_engine_set_mark (e);
 	html_engine_end_of_line (e);
 }
+
+struct SetData {
+	HTMLType      object_type;
+	const gchar  *key;
+	gpointer      value;
+};
+
+static void
+set_data (HTMLObject *o, HTMLEngine *e, gpointer p)
+{
+	struct SetData *data = (struct SetData *) p;
+
+	if (HTML_OBJECT_TYPE (o) == data->object_type) {
+		/* printf ("set data %s --> %p\n", data->key, data->value); */
+		html_object_set_data (o, data->key, data->value);
+	}
+}
+
+void
+html_engine_set_data_by_type (HTMLEngine *e, HTMLType object_type, const gchar *key, gpointer value)
+{
+	struct SetData *data = g_new (struct SetData, 1);
+
+	/* printf ("html_engine_set_data_by_type %s\n", key); */
+
+	data->object_type = object_type;
+	data->key         = key;
+	data->value       = value;
+
+	html_object_forall (e->clue, NULL, set_data, data);
+
+	g_free (data);
+}
