@@ -33,4 +33,6 @@ void  html_engine_print_with_header_footer  (HTMLEngine           *e,
 					     gdouble               footer_height,
 					     GtkHTMLPrintCallback  header_print,
 					     GtkHTMLPrintCallback  footer_print);
+void  html_engine_print_set_min_split_index (HTMLEngine           *e,
+					     gdouble               idx);
 #endif
