@@ -24,29 +24,13 @@
 #ifndef _HTMLENGINE_H_
 #define _HTMLENGINE_H_
 
-#include <glib.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#ifdef GTKHTML_HAVE_PSPELL
-#include <pspell/pspell.h>
-#endif
-
-typedef struct _HTMLEngine HTMLEngine;
-typedef struct _HTMLEngineClass HTMLEngineClass;
-#include "gtkhtml.h"
+#include <gdk/gdktypes.h>
+#include <gtk/gtkobject.h>
+#include <gtk/gtkwidget.h>
+#include "gtkhtml-types.h"
 
-#include "htmltokenizer.h"
-#include "htmlcursor.h"
-#include "htmldrawqueue.h"
-#include "htmlstack.h"
-#include "htmlsettings.h"
-#include "htmlpainter.h"
-#include "htmlgdkpainter.h"
-#include "htmlundo.h"
-#include "htmlstringtokenizer.h"
-#include "htmlengine-edit-selection-updater.h"
-#include "htmlreplace.h"
-
-#include "gtkhtml-embedded.h"
+#include "htmltypes.h"
+#include "htmlenums.h"
 
 
 #define HTML_TYPE_ENGINE                 (html_engine_get_type ())
@@ -63,15 +47,6 @@ typedef struct _HTMLEngineClass HTMLEngineClass;
 #define TOP_BORDER 10
 #define BOTTOM_BORDER 10
 
-enum _HTMLGlossaryEntry {
-	HTML_GLOSSARY_DL = 1,
-	HTML_GLOSSARY_DD = 2
-};
-typedef enum _HTMLGlossaryEntry HTMLGlossaryEntry;
-
-typedef struct _HTMLBlockStackElement HTMLBlockStackElement;
-
-
 /* FIXME this needs splitting.  */
 
 struct _HTMLEngine {
@@ -129,6 +104,7 @@ struct _HTMLEngine {
 	gboolean inPre;
 	gboolean inOption;
 	gboolean inTextArea;
+	gboolean eat_space;
 
 	gboolean newPage;
  
@@ -201,6 +177,9 @@ struct _HTMLEngine {
 	/* Whether we have the keyboard focus.  */
 	guint have_focus : 1;
 
+	HTMLInterval *selection;
+	gboolean selected_in;
+
 	/* --- */
 
 	/* Editing stuff.  -- FIXME it should be in a separate object.  */
@@ -230,9 +209,7 @@ struct _HTMLEngine {
 	gchar            *insertion_url;
 	gchar            *insertion_target;
 
-	/* This is set to TRUE when at least one element is selected (in whole
-           or in part), to FALSE when no item is selected at all.  */
-	gboolean active_selection;
+	/* if we are doing shift selection - Shift + some event (arrows, mouse motion, ...) */
 	gboolean shift_selection;
 
 	/* This object is used to update the keyboard selection in the
@@ -247,14 +224,10 @@ struct _HTMLEngine {
 	HTMLSearch  *search_info;
 	HTMLReplace *replace_info;
 
-#ifdef GTKHTML_HAVE_PSPELL
-	/* spell checking */
-	PspellConfig  *spell_config;
-	PspellManager *spell_checker;
-#endif
-	
-	/* image id storage */
-	GHashTable *imageid_table;
+	/* id-to-object mapping */
+	GHashTable *id_table;
+
+	gdouble min_split_index;
 };
 
 /* must be forward referenced *sigh* */
@@ -293,7 +266,8 @@ void  html_engine_set_focus  (HTMLEngine *engine,
 			      gboolean    have_focus);
 
 /* Parsing control.  */
-GtkHTMLStream *html_engine_begin            (HTMLEngine  *p);
+GtkHTMLStream *html_engine_begin            (HTMLEngine  *p, 
+					     char        *content_type);
 void           html_engine_parse            (HTMLEngine  *p);
 void           html_engine_stop_parser      (HTMLEngine  *e);
 
@@ -338,6 +312,10 @@ HTMLObject  *html_engine_get_object_at  (HTMLEngine *e,
 					 gint        y,
 					 guint      *offset_return,
 					 gboolean    for_cursor);
+HTMLPoint   *html_engine_get_point_at   (HTMLEngine *e,
+					 gint        x,
+					 gint        y,
+					 gboolean    for_cursor);
 const gchar *html_engine_get_link_at    (HTMLEngine *e,
 					 gint        x,
 					 gint        y);
@@ -352,28 +330,9 @@ void  html_engine_form_submitted  (HTMLEngine  *engine,
 gchar *html_engine_canonicalize_url  (HTMLEngine *e,
 				      const char *in_url);
 
-/* Selection.  (FIXME: Maybe `htmlengine-edit' instead?)  */
-void   html_engine_select_region         (HTMLEngine *e,
-					  gint        x1,
-					  gint        y1,
-					  gint        x2,
-					  gint        y2);
-
-void   html_engine_set_active_selection  (HTMLEngine *e,
-					  gboolean active,
-					  guint32 time);
-HTMLCursor *
-       html_engine_get_cursor            (HTMLEngine *e);
-void   html_engine_select_word           (HTMLEngine *e);
-void   html_engine_select_line           (HTMLEngine *e);
-void   html_engine_unselect_all          (HTMLEngine *e);
-
-void   html_engine_disable_selection     (HTMLEngine *e);
-gchar *html_engine_get_selection_string  (HTMLEngine *e);
-
-/* Cursor normalization.  */
-
-void  html_engine_normalize_cursor  (HTMLEngine *e);
+/* Cursor */
+HTMLCursor *html_engine_get_cursor        (HTMLEngine *e);
+void        html_engine_normalize_cursor  (HTMLEngine *e);
 
 /* Freezing/thawing.  */
 gboolean  html_engine_frozen  (HTMLEngine *engine);
@@ -398,14 +357,23 @@ gint      html_engine_replaced                  (void);
 /* Magic links */
 void      html_engine_init_magic_links          (void);
 
-#ifdef GTKHTML_HAVE_PSPELL
 /* spell checking */
-void      html_engine_spell_check               (HTMLEngine *e);
-void      html_engine_spell_check_word          (HTMLEngine *e);
-gchar *   html_engine_get_word                  (HTMLEngine *e);
-gboolean  html_engine_word_is_valid             (HTMLEngine *e);
-void      html_engine_replace_word_with         (HTMLEngine *e,
-						 const gchar *word);
-#endif
+void      html_engine_spell_check        (HTMLEngine  *e);
+gchar    *html_engine_get_word           (HTMLEngine  *e);
+gboolean  html_engine_word_is_valid      (HTMLEngine  *e);
+void      html_engine_replace_word_with  (HTMLEngine  *e,
+					  const gchar *word);
 
+/* view size - for image size specified in percent */
+gint  html_engine_get_view_width   (HTMLEngine *e);
+gint  html_engine_get_view_height  (HTMLEngine *e);
+
+/* id support */
+void        html_engine_add_object_with_id  (HTMLEngine  *e,
+					     const gchar *id,
+					     HTMLObject  *obj);
+HTMLObject *html_engine_get_object_by_id    (HTMLEngine  *e,
+					     const gchar *id);
+
+HTMLEngine *html_engine_get_top_html_engine (HTMLEngine *e);
 #endif /* _HTMLENGINE_H_ */
