@@ -19,15 +19,18 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libart_lgpl/art_rect.h>
-#include <gdk/gdkx.h>
 #include <unicode.h>
+#include <gdk/gdkx.h>
+#include <libart_lgpl/art_rect.h>
 #include <gal/widgets/e-font.h>
 
 #include "htmlentity.h"
 #include "htmlgdkpainter.h"
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
 
 
 static HTMLPainterClass *parent_class = NULL;
@@ -75,7 +78,7 @@ get_size (gchar *font_name)
 }
 
 static gboolean
-find_font (gchar *font_name, gdouble req_size, gint *font_size)
+find_font (gchar *font_name, gdouble req_size, gint *font_size, GtkHTMLFontStyle style)
 {
 	gint n;
 	gdouble size, smaller, bigger;
@@ -107,7 +110,8 @@ find_font (gchar *font_name, gdouble req_size, gint *font_size)
 		if (smaller == .0)
 			size = bigger;
 		if (bigger != .0 && smaller != .0)
-			size = (bigger - req_size <= req_size - smaller) ? bigger : smaller;
+			size = ((style & GTK_HTML_FONT_STYLE_SIZE_MASK) > GTK_HTML_FONT_STYLE_SIZE_3) ? bigger : smaller;
+		/* size = (bigger - req_size <= req_size - smaller) ? bigger : smaller; */
 		*font_size = size;
 		rv = TRUE;
 	}
@@ -115,65 +119,79 @@ find_font (gchar *font_name, gdouble req_size, gint *font_size)
 	return rv;
 }
 
-static GdkFont *
-get_font_with_enc (const gchar *family, const gchar *weight, const gchar *slant, gdouble size, gchar *enc)
+static void
+gtkhtml_private_split_name (gchar * c[], gchar * name)
 {
-	GdkFont *font;
-	gchar *font_name;
-	gint font_size;
+	gchar *p;
+	gint i;
 
-	font      = NULL;
-	font_name = g_strdup_printf ("-*-%s-%s-%s-normal-*-*-*-*-*-*-*-%s",
-				     family, weight, slant, enc);
-	if (find_font (font_name, size, &font_size)) {
-		g_free (font_name);
-		font_name = g_strdup_printf ("-*-%s-%s-%s-normal-*-%d-*-*-*-*-*-%s",
-					     family, weight, slant, font_size, enc);
-		font = gdk_font_load (font_name);
+	p = name;
+
+	for (i = 0; i < 13; i++) {
+		c[i] = p;
+		/* Skip text */
+		while (*p && (*p != '-')) p++;
+		/* Replace hyphen with '\0' */
+		if (*p) *p++ = '\0';
 	}
-	g_free (font_name);
 
-	/* printf ("family: %s enc: %s ==> %p\n", family, enc, font); */
-
-	return font;
-}
-
-static GdkFont *
-get_font (const gchar *family, const gchar *weight, const gchar *slant, gdouble size)
-{
-	GdkFont *font;
-
-	/* prefer unicode fonts */
-	font = get_font_with_enc (family, weight, slant, size, "iso10646-1");
-	return font ? font : get_font_with_enc (family, weight, slant, size, "*-*");
-}
-
-static GdkFont *
-get_closest_font (const gchar *family, const gchar *weight, const gchar *slant, gdouble size)
-{
-	GdkFont *font;
-
-	font = get_font (family, weight, slant, size);
-	if (!font && *slant == 'i')
-		font = get_font (family, weight, "o", size);
-
-	return font;
+	c[i] = p;
 }
 
 static gpointer
-alloc_font (gchar *face, gdouble size, GtkHTMLFontStyle style)
+alloc_e_font_it (gchar *face, gdouble size, GtkHTMLFontStyle style, gchar *it)
 {
-	return (face) ? get_closest_font (face, (style & GTK_HTML_FONT_STYLE_BOLD) ? "bold" : "medium",
-					  (style & GTK_HTML_FONT_STYLE_ITALIC) ? "i" : "r", size) : gdk_font_load ("fixed");
+	EFont *font;
+	gchar *name;
+
+
+	if (face) {
+		gint tsize;
+
+		name = g_strdup_printf ("-*-%s-%s-%s-normal-*-*-*-*-*-*-*-*",
+					face, style & GTK_HTML_FONT_STYLE_BOLD ? "bold" : "medium",
+					style & GTK_HTML_FONT_STYLE_ITALIC ? it : "r");
+		if (!find_font (name, size, &tsize, style))
+			tsize = size;
+		g_free (name);
+		name = g_strdup_printf ("-*-%s-%s-%s-normal-*-%d-*-*-*-*-*-*",
+					face, style & GTK_HTML_FONT_STYLE_BOLD ? "bold" : "medium",
+					style & GTK_HTML_FONT_STYLE_ITALIC ? it : "r", tsize);
+	} else
+		name = g_strdup ("fixed");
+
+	font = e_font_from_gdk_name (name);
+	g_free (name);
+
+	if (face) {
+		gchar *c [14];
+
+		name = e_font_get_name (font);
+		if (name) {
+			gtkhtml_private_split_name (c, name);
+			if (!strcasecmp (c [2], "fixed") && strcasecmp (c [2], face)) {
+				e_font_unref (font);
+				font = NULL;
+			}
+			g_free (name);
+		}
+	}
+
+	return font;
 }
 
 static gpointer
 alloc_e_font (gchar *face, gdouble size, GtkHTMLFontStyle style)
 {
-	GdkFont *font;
+	
 
-	font = alloc_font (face, size, style);
-	return font ? e_font_from_gdk_font (font) : NULL;
+	EFont *font;
+
+	font = alloc_e_font_it (face, size, style, "i");
+	if (!font && style & GTK_HTML_FONT_STYLE_ITALIC)
+		font = alloc_e_font_it (face, size, style, "o");
+
+	return font;
 }
 
 static void
@@ -387,6 +405,7 @@ draw_rect (HTMLPainter *painter,
 
 static void
 draw_panel (HTMLPainter *painter,
+	    GdkColor *bg,
 	    gint x, gint y,
 	    gint width, gint height,
 	    GtkHTMLEtchStyle inset,
@@ -394,23 +413,11 @@ draw_panel (HTMLPainter *painter,
 {
 	HTMLGdkPainter *gdk_painter;
 	GdkColor *col1 = NULL, *col2 = NULL;
-	GdkColor *bg, dark, light;
-	gint value, ldelta, ddelta;
+	GdkColor dark, light;
 
-#define DARK(c) 	dark. ## c  = MIN (MAX (((gint) bg-> ## c) + (gint) ddelta*(((gdouble) bg-> ## c)/0xffff), 0), 0xffff);
-#define LIGHT(c)	light. ## c = MIN (MAX (((gint) bg-> ## c) + (gint) ldelta*(((gdouble) bg-> ## c)/0xffff), 0), 0xffff);
-#define THOLD 0x2aaa
-
-	bg = &((html_colorset_get_color (painter->color_set, HTMLBgColor))->color);
-
-	value = MAX (bg->red, MAX (bg->green, bg->blue));
-	if (value > 0x7fff) {
-		ldelta = -THOLD;
-		ddelta = -3*THOLD;
-	} else {
-		ldelta = 3*THOLD;
-		ddelta = THOLD;
-	}
+	#define INC 0x8000
+	#define DARK(c)  dark.  ## c = MAX (((gint) bg-> ## c) - INC, 0)
+	#define LIGHT(c) light. ## c = MIN (((gint) bg-> ## c) + INC, 0xffff)
 
 	DARK(red);
 	DARK(green);
@@ -882,7 +889,6 @@ e_style (GtkHTMLFontStyle style)
 	return rv;
 }
 
-#ifdef GTKHTML_HAVE_PSPELL
 static void
 draw_spell_error (HTMLPainter *painter,
 		  gint x, gint y,
@@ -922,7 +928,6 @@ draw_spell_error (HTMLPainter *painter,
 	gdk_gc_set_line_attributes (gdk_painter->gc, values.line_width,
 				    values.line_style, values.cap_style, values.join_style);
 }
-#endif
 
 static void
 draw_text (HTMLPainter *painter,
@@ -1023,12 +1028,16 @@ calc_text_width (HTMLPainter *painter,
 	EFont *e_font;
 	gint width;
 
+
 	gdk_painter = HTML_GDK_PAINTER (painter);
 	e_font = html_painter_get_font (painter, face, style);
 
-	width = e_font_utf8_text_width (e_font, e_style (painter->font_style),
+	width = e_font_utf8_text_width (e_font, e_style (style),
 					text, unicode_offset_to_index (text, len));
 
+	/* printf ("calc_text_width text: %s len: %d\n", text, len);
+	{ gint i; printf ("["); for (i=0;i<unicode_offset_to_index (text, len); i++) printf ("%c", text [i]); printf ("] ");}
+	printf ("%d (%p)\n", width, e_font); */
 	return width;
 }
 
@@ -1096,9 +1105,7 @@ class_init (GtkObjectClass *object_class)
 	painter_class->draw_line = draw_line;
 	painter_class->draw_rect = draw_rect;
 	painter_class->draw_text = draw_text;
-#ifdef GTKHTML_HAVE_PSPELL
 	painter_class->draw_spell_error = draw_spell_error;
-#endif
 	painter_class->fill_rect = fill_rect;
 	painter_class->draw_pixmap = draw_pixmap;
 	painter_class->draw_ellipse = draw_ellipse;
