@@ -1,3 +1,4 @@
+
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* This file is part of the GtkHTML library.
    
@@ -18,11 +19,19 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
+
+#include <config.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#include "gtkhtml.h"
+#include "gtkhtml-private.h"
+#include "htmlcolorset.h"
+#include "htmlgdkpainter.h"
 #include "htmliframe.h"
 #include "htmlengine-search.h"
 #include "htmlsearch.h"
+#include "htmlselection.h"
+#include "htmlsettings.h"
 
 HTMLIFrameClass html_iframe_class;
 static HTMLEmbeddedClass *parent_class = NULL;
@@ -65,7 +74,7 @@ iframe_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
 }
 
 static void
-iframe_size_changed (GtkAdjustment *adj, gpointer data)
+iframe_size_changed (GtkHTML *html, gpointer data)
 {
 	HTMLIFrame *iframe = HTML_IFRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
@@ -148,6 +157,15 @@ reset (HTMLObject *o)
 	html_object_reset (GTK_HTML (iframe->html)->engine->clue);
 }
 
+static void
+draw_background (HTMLObject *self,
+		 HTMLPainter *p,
+		 gint x, gint y,
+		 gint width, gint height,
+		 gint tx, gint ty)
+{
+}
+
 /* FIXME rodo - draw + set_painter is not much clean now, needs refactoring */
 
 static void
@@ -175,9 +193,11 @@ draw (HTMLObject *o,
 		new_x = GTK_LAYOUT (element->parent)->hadjustment->value + o->x + tx;
 		new_y = GTK_LAYOUT (element->parent)->vadjustment->value + o->y + ty - o->ascent;
 
-		if(new_x != element->abs_x || new_y != element->abs_y)
+		if(new_x != element->abs_x || new_y != element->abs_y) {
+			printf ("moveto: %d,%d\n", new_x, new_y);
 			gtk_layout_move(GTK_LAYOUT(element->parent), element->widget,
 					new_x, new_y);
+		}
 		element->abs_x = new_x;
 		element->abs_y = new_y;
 	}
@@ -198,14 +218,15 @@ set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
 
 static void
 forall (HTMLObject *self,
+	HTMLEngine *e,
 	HTMLObjectForallFunc func,
 	gpointer data)
 {
 	HTMLIFrame *iframe;
 
 	iframe = HTML_IFRAME (self);
-	(* func) (self, data);
-	html_object_forall (GTK_HTML (iframe->html)->engine->clue, func, data);
+	(* func) (self, html_object_get_engine (self, e), data);
+	html_object_forall (GTK_HTML (iframe->html)->engine->clue, html_object_get_engine (self, e), func, data);
 }
 
 static gint
@@ -287,6 +308,75 @@ search (HTMLObject *self, HTMLSearch *info)
 	return FALSE;
 }
 
+static HTMLObject *
+head (HTMLObject *self)
+{
+	return GTK_HTML (HTML_IFRAME (self)->html)->engine->clue;
+}
+
+static HTMLObject *
+tail (HTMLObject *self)
+{
+	return GTK_HTML (HTML_IFRAME (self)->html)->engine->clue;
+}
+
+static HTMLEngine *
+get_engine (HTMLObject *self, HTMLEngine *e)
+{
+	return GTK_HTML (HTML_IFRAME (self)->html)->engine;
+}
+
+static HTMLObject*
+check_point (HTMLObject *self,
+	     HTMLPainter *painter,
+	     gint x, gint y,
+	     guint *offset_return,
+	     gboolean for_cursor)
+{
+	HTMLEngine *e = GTK_HTML (HTML_IFRAME (self)->html)->engine;
+
+	if (x < self->x || x >= self->x + self->width
+	    || y >= self->y + self->descent || y < self->y - self->ascent)
+		return NULL;
+
+	x -= self->x + e->leftBorder;
+	y -= self->y - self->ascent + e->topBorder;
+
+	if (for_cursor && (x < 0 || y < e->clue->y - e->clue->ascent))
+		return html_object_check_point (e->clue, e->painter, 0, e->clue->y - e->clue->ascent,
+						offset_return, for_cursor);
+
+	if (for_cursor && (x > e->clue->width - 1 || y > e->clue->y + e->clue->descent - 1))
+		return html_object_check_point (e->clue, e->painter, e->clue->width - 1, e->clue->y + e->clue->descent - 1,
+						offset_return, for_cursor);
+
+	return html_object_check_point (e->clue, e->painter, x, y, offset_return, for_cursor);
+}
+
+static gboolean
+is_container (HTMLObject *self)
+{
+	return TRUE;
+}
+
+static void
+append_selection_string (HTMLObject *self,
+			 GString *buffer)
+{
+	html_object_append_selection_string (GTK_HTML (HTML_IFRAME (self)->html)->engine->clue, buffer);
+}
+
+/* static gboolean
+select_range (HTMLObject *self,
+	      HTMLEngine *engine,
+	      guint start,
+	      gint length,
+	      gboolean queue_draw)
+{
+	return html_object_select_range (GTK_HTML (HTML_IFRAME (self)->html)->engine->clue,
+					 GTK_HTML (HTML_IFRAME (self)->html)->engine,
+					 start, length, queue_draw);
+} */
 
 void 
 html_iframe_init (HTMLIFrame *iframe,
@@ -298,12 +388,14 @@ html_iframe_init (HTMLIFrame *iframe,
 		  gboolean border)
 {
 	HTMLEmbedded *em = HTML_EMBEDDED (iframe);
-	GtkWidget *html;
+	GtkWidget *new_widget;
+	GtkHTML   *new_html;
 	GtkHTML   *parent_html;
 	GtkHTMLStream *handle;
 	GtkWidget *scrolled_window;
 
 	g_assert (GTK_IS_HTML (parent));
+	parent_html = GTK_HTML (parent);
 
 	html_embedded_init (em, HTML_EMBEDDED_CLASS (klass),
 			    parent, NULL, NULL);
@@ -312,65 +404,61 @@ html_iframe_init (HTMLIFrame *iframe,
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
-	html = gtk_html_new ();
-	iframe->html = html;
-	/* GTK_HTML (html)->engine->clue->parent = HTML_OBJECT (iframe); */
-	gtk_html_set_iframe_parent (GTK_HTML (html), parent);
-	gtk_container_add (GTK_CONTAINER (scrolled_window), html);
-	gtk_widget_show (html);
+	new_widget = gtk_html_new ();
+	new_html = GTK_HTML (new_widget);
+
+	gtk_html_set_default_content_type (new_html,
+					   parent_html->priv->content_type);
+	iframe->html = new_widget;
+	gtk_html_set_iframe_parent (new_html, parent);
+	gtk_container_add (GTK_CONTAINER (scrolled_window), new_widget);
+	gtk_widget_show (new_widget);
 
 	iframe->url = src;
 	iframe->width = width;
 	iframe->height = height;
 
-	parent_html = GTK_HTML (parent);
+	handle = gtk_html_begin (new_html);
+	new_html->engine->clue->parent = HTML_OBJECT (iframe);
 
-	handle = gtk_html_begin (GTK_HTML (html));
-
-	gtk_signal_connect (GTK_OBJECT (html), "url_requested",
+	gtk_signal_connect (GTK_OBJECT (new_html), "url_requested",
 			    GTK_SIGNAL_FUNC (iframe_url_requested),
 			    (gpointer)iframe);
-	gtk_signal_connect (GTK_OBJECT (html), "on_url",
+	gtk_signal_connect (GTK_OBJECT (new_html), "on_url",
 			    GTK_SIGNAL_FUNC (iframe_on_url), 
 			    (gpointer)iframe);
-	gtk_signal_connect (GTK_OBJECT (html), "link_clicked",
+	gtk_signal_connect (GTK_OBJECT (new_html), "link_clicked",
 			    GTK_SIGNAL_FUNC (iframe_link_clicked),
 			    (gpointer)iframe);	
-	gtk_signal_connect (GTK_OBJECT (html), "size_changed",
+	gtk_signal_connect (GTK_OBJECT (new_html), "size_changed",
 			    GTK_SIGNAL_FUNC (iframe_size_changed),
 			    (gpointer)iframe);	
-	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
+	gtk_signal_connect (GTK_OBJECT (new_html), "object_requested",
 			    GTK_SIGNAL_FUNC (iframe_object_requested),
 			    (gpointer)iframe);	
 	/*
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (iframe_button_press_event), iframe);
 	*/
-	gtk_signal_emit_by_name (GTK_OBJECT (GTK_HTML (html)->engine), 
+	gtk_signal_emit_by_name (GTK_OBJECT (new_html->engine), 
 				 "url_requested", src, handle);
 
 	if ((width > 0) && (height > 0))
 		gtk_widget_set_usize (scrolled_window, width, height);
 
-	/* 
-	   FIXME: huge hack to get the widget to allocate it's size 
-	   before it is actually drawn.  Place it in the gtklayout but offscreen
-	   it will be moved to the proper place in the draw routine, but it's size
-	   allocation needs to happen now.
-	*/
-	gtk_widget_show (scrolled_window);
-	
+	gtk_widget_show (scrolled_window);	
 	iframe->scroll = scrolled_window;
-	html_embedded_set_widget (em, scrolled_window);
-	html_embedded_size_recalc(em);
 
-	gtk_signal_connect(GTK_OBJECT(scrolled_window), "button_press_event",
-			   GTK_SIGNAL_FUNC(html_iframe_grab_cursor), NULL);
+	html_embedded_set_widget (em, scrolled_window);
+	html_embedded_size_recalc (em);
+
+	gtk_signal_connect(GTK_OBJECT (scrolled_window), "button_press_event",
+			   GTK_SIGNAL_FUNC (html_iframe_grab_cursor), NULL);
 
 	/* inherit the current colors from our parent */
-	html_colorset_set_unchanged (GTK_HTML (html)->engine->defaultSettings->color_set,
+	html_colorset_set_unchanged (new_html->engine->defaultSettings->color_set,
 				     parent_html->engine->settings->color_set);
-	html_colorset_set_unchanged (GTK_HTML (html)->engine->settings->color_set,
+	html_colorset_set_unchanged (new_html->engine->settings->color_set,
 				     parent_html->engine->settings->color_set);
 	/*
 	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
@@ -411,13 +499,20 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	html_embedded_class_init (embedded_class, type, size);
 	parent_class = &html_embedded_class;
 
-	object_class->calc_size = calc_size;
-	object_class->calc_min_width = calc_min_width;
-	object_class->set_painter = set_painter;
-	object_class->reset = reset;
-	object_class->draw = draw;
-	object_class->set_max_width = set_max_width;
-	object_class->forall = forall;
-	object_class->check_page_split = check_page_split;
-	object_class->search = search;
+	object_class->calc_size               = calc_size;
+	object_class->calc_min_width          = calc_min_width;
+	object_class->set_painter             = set_painter;
+	object_class->reset                   = reset;
+	object_class->draw                    = draw;
+	object_class->set_max_width           = set_max_width;
+	object_class->forall                  = forall;
+	object_class->check_page_split        = check_page_split;
+	object_class->search                  = search;
+	object_class->head                    = head;
+	object_class->tail                    = tail;
+	object_class->get_engine              = get_engine;
+	object_class->check_point             = check_point;
+	object_class->is_container            = is_container;
+	object_class->draw_background         = draw_background;
+	object_class->append_selection_string = append_selection_string;
 }
