@@ -31,19 +31,23 @@
 
 */
 
+#include <config.h>
 #include <glib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include "htmlobject.h"
-#include "htmlimage.h"
-#include "htmlengine.h"
-#include "htmlpainter.h"
-#include "htmlprinter.h"
-
-#include "gtkhtml-private.h"
+#include "gtkhtml.h"
+#include "gtkhtml-properties.h"
 #include "gtkhtml-stream.h"
 
-
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
+#include "htmlengine.h"
+#include "htmlengine-save.h"
+#include "htmlenumutils.h"
+#include "htmlimage.h"
+#include "htmlobject.h"
+#include "htmlprinter.h"
+
 /* HTMLImageFactory stuff.  */
 
 struct _HTMLImageFactory {
@@ -65,7 +69,8 @@ static HTMLImagePointer   *html_image_pointer_new       (const char *filename, H
 static void                html_image_pointer_ref       (HTMLImagePointer *ip);
 static void                html_image_pointer_unref     (HTMLImagePointer *ip);
 
-static void render_cur_frame (HTMLImage *image, gint nx, gint ny, const GdkColor *highlight_color);
+static void                render_cur_frame             (HTMLImage *image, gint nx, gint ny, const GdkColor *highlight_color);
+static guint               get_actual_height            (HTMLImage *image, HTMLPainter *painter);
 
 
 static guint
@@ -76,23 +81,23 @@ get_actual_width (HTMLImage *image,
 	GdkPixbufAnimation *anim = image->image_ptr->animation;
 	gint width;
 
-	if (image->specified_width > 0) {
-		width = image->specified_width * html_painter_get_pixel_size (painter);
-	} else if (HTML_OBJECT (image)->percent > 0) {
+	if (image->percent_width) {
 		/* The cast to `gdouble' is to avoid overflow (eg. when
                    printing).  */
-		width = ((gdouble) HTML_OBJECT (image)->max_width
-			 * HTML_OBJECT (image)->percent) / 100;
+		width = ((gdouble) html_engine_get_view_width (image->image_ptr->factory->engine)
+			 * image->specified_width) / 100;
+	} else if (image->specified_width > 0) {
+		width = image->specified_width * html_painter_get_pixel_size (painter);
 	} else if (image->image_ptr == NULL || pixbuf == NULL) {
 		width = DEFAULT_SIZE * html_painter_get_pixel_size (painter);
 	} else {
 		width = (((anim) ? gdk_pixbuf_animation_get_width (anim) : gdk_pixbuf_get_width (pixbuf))
 			  * html_painter_get_pixel_size (painter));
 
-		if (image->specified_height > 0) {
+		if (image->specified_height > 0 || image->percent_height) {
 			double scale;
 
-			scale =  ((double)image->specified_height) 
+			scale =  ((double) get_actual_height (image, painter)) 
 				/ ((anim) ? gdk_pixbuf_animation_get_height (anim) : gdk_pixbuf_get_height (pixbuf));
 			
 			width *= scale;
@@ -112,7 +117,12 @@ get_actual_height (HTMLImage *image,
 	GdkPixbufAnimation *anim = image->image_ptr->animation;
 	gint height;
 		
-	if (image->specified_height > 0) {
+	if (image->percent_height) {
+		/* The cast to `gdouble' is to avoid overflow (eg. when
+                   printing).  */
+		height = ((gdouble) html_engine_get_view_height (image->image_ptr->factory->engine)
+			  * image->specified_height) / 100;
+	} else if (image->specified_height > 0) {
 		height = image->specified_height * html_painter_get_pixel_size (painter);
 	} else if (image->image_ptr == NULL || pixbuf == NULL) {
 		height = DEFAULT_SIZE * html_painter_get_pixel_size (painter);
@@ -120,10 +130,10 @@ get_actual_height (HTMLImage *image,
 		height = (((anim) ? gdk_pixbuf_animation_get_height (anim) : gdk_pixbuf_get_height (pixbuf))
 			  * html_painter_get_pixel_size (painter));
 
-		if ((image->specified_width > 0) || (HTML_OBJECT(image)->percent > 0)) {
+		if (image->specified_width > 0 || image->percent_width) {
 			double scale;
 			
-			scale = ((double)get_actual_width (image, painter))
+			scale = ((double) get_actual_width (image, painter))
 				/ (((anim) ? gdk_pixbuf_animation_get_width (anim) : gdk_pixbuf_get_width (pixbuf))
 				   * html_painter_get_pixel_size (painter));
 			
@@ -154,6 +164,7 @@ destroy (HTMLObject *o)
 
 	if (image->url)    g_free (image->url);
 	if (image->target) g_free (image->target);
+	if (image->alt)    g_free (image->alt);
 
 	if (image->color)
 		html_color_unref (image->color);
@@ -173,20 +184,29 @@ copy (HTMLObject *self,
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
 
 	dimg->image_ptr = simg->image_ptr;
-	dimg->border = simg->border;
-	dimg->specified_width = simg->specified_width;
-	dimg->specified_height = simg->specified_height;
-	dimg->url = g_strdup (simg->url);
-	dimg->target = g_strdup (simg->target);
+	dimg->animation = NULL;          /* don't bother with animation copying now. TODO */
 	dimg->color = simg->color;
 	if (simg->color)
 		html_color_ref (dimg->color);
+
 	dimg->have_color = simg->have_color;
-	dimg->valign = simg->valign;
-	dimg->animation = NULL;          /* don't bother with animation copying now. TODO */
+
+	dimg->border = simg->border;
+
+	dimg->specified_width = simg->specified_width;
+	dimg->specified_height = simg->specified_height;
+	dimg->percent_width = simg->percent_width;
+	dimg->percent_height = simg->percent_height;
+
 	dimg->hspace = simg->hspace;
 	dimg->vspace = simg->vspace;
 
+	dimg->valign = simg->valign;
+
+	dimg->url = g_strdup (simg->url);
+	dimg->target = g_strdup (simg->target);
+	dimg->alt = g_strdup (simg->alt);
+
 	/* add dest to image_ptr interests */
 	dimg->image_ptr->interests = g_slist_prepend (dimg->image_ptr->interests, dimg);
 }
@@ -201,7 +221,7 @@ calc_min_width (HTMLObject *o,
 
 	pixel_size = html_painter_get_pixel_size (painter);
 
-	if (o->percent > 0)
+	if (image->percent_width)
 		min_width = pixel_size;
 	else
 		min_width = get_actual_width (HTML_IMAGE (o), painter);
@@ -301,6 +321,7 @@ draw (HTMLObject *o,
 						o->ascent + o->descent - 2 * vspace);
 		}
 		html_painter_draw_panel (painter,
+					 &((html_colorset_get_color (painter->color_set, HTMLBgColor))->color),
 					 o->x + tx + hspace,
 					 o->y + ty - o->ascent + vspace,
 					 o->width - 2 * hspace,
@@ -321,7 +342,8 @@ draw (HTMLObject *o,
 			html_painter_set_pen (painter, &image->color->color);
 		}
 		
-		html_painter_draw_panel (painter, 
+		html_painter_draw_panel (painter,
+					 &((html_colorset_get_color (painter->color_set, HTMLBgColor))->color),
 					 base_x - image->border * pixel_size,
 					 base_y - image->border * pixel_size,
 					 scale_width + (2 * image->border) * pixel_size,
@@ -345,30 +367,65 @@ draw (HTMLObject *o,
 	}
 }
 
+gchar *
+html_image_resolve_image_url (GtkHTML *html, gchar *image_url)
+{
+	gchar *url = NULL;
+
+	/* printf ("html_image_resolve_image_url %p\n", html->editor_api); */
+	if (html->editor_api) {
+		GtkArg *args [2];
+
+		args [0] = gtk_arg_new (GTK_TYPE_STRING);
+		GTK_VALUE_STRING (*args [0]) = image_url;
+
+		args [1] = (* html->editor_api->event) (html, GTK_HTML_EDITOR_EVENT_IMAGE_URL, args, html->editor_data);
+		gtk_arg_free (args [0], FALSE);
+
+		if (args [1]) {
+			if (args [1]->type == GTK_TYPE_STRING)
+				url = GTK_VALUE_STRING (*args [1]);
+			gtk_arg_free (args [1], FALSE);
+		}
+	}
+	if (!url)
+		url = g_strdup (image_url);
+	/* printf ("image URL resolved to: %s (from: %s)\n", url, image_url); */
+
+	return url;
+}
+
 static gboolean
 save (HTMLObject *self,
       HTMLEngineSaveState *state)
 {
 	HTMLImage *image;
+	gchar *url;
+	gboolean result;
 
 	g_return_val_if_fail (self != NULL, FALSE);
 	g_return_val_if_fail (state != NULL, FALSE);
-	
-	image = HTML_IMAGE (self);
-	
-	if (!html_engine_save_output_string (state, "<IMG SRC=\"%s\"", image->image_ptr->url))
-	        return FALSE;	
 
-	if (image->specified_width > 0) {
-		if (!html_engine_save_output_string (state, " WIDTH=\"%d\"", image->specified_height))
+	image  = HTML_IMAGE (self);
+
+	url    = html_image_resolve_image_url (state->engine->widget, image->image_ptr->url);
+	result = html_engine_save_output_string (state, "<IMG SRC=\"%s\"", url);
+	g_free (url);
+	if (!result)
+		return FALSE;	
+
+	if (image->percent_width) {
+		if (!html_engine_save_output_string (state, " WIDTH=\"%d\%\"", image->specified_width))
 			return FALSE;
-	} else if (self->percent) {
-		if (!html_engine_save_output_string (state, " WIDTH=\"%d\%\"", self->percent))
+	} else if (image->specified_width > 0) {
+		if (!html_engine_save_output_string (state, " WIDTH=\"%d\"", image->specified_width))
 			return FALSE;
 	}
 
-	/* FIXME percent heights are supported in netscape/mozilla */
-	if (image->specified_height > 0) {
+	if (image->percent_height) {
+		if (!html_engine_save_output_string (state, " HEIGHT=\"%d\%\"", image->specified_height))
+			return FALSE;
+	} else if (image->specified_height > 0) {
 		if (!html_engine_save_output_string (state, " HEIGHT=\"%d\"", image->specified_height))
 			return FALSE;
 	}
@@ -388,6 +445,16 @@ save (HTMLObject *self,
 			return FALSE;
 	}
 
+	if (image->valign != HTML_VALIGN_NONE) {
+		if (!html_engine_save_output_string (state, " ALIGN=\"%s\"", html_valign_name (image->valign)))
+			return FALSE;
+	}
+
+	if (image->alt) {
+		if (!html_engine_save_output_string (state, " ALT=\"%s\"", image->alt))
+			return FALSE;
+	}
+
 	/* FIXME this is the default set in htmlengine.c but there is no real way to tell
 	 * if the usr specified it directly
 	 */
@@ -396,13 +463,28 @@ save (HTMLObject *self,
 			return FALSE;
 	}
 
-	/* FIXME we are not preserving alt tags */
 	if (!html_engine_save_output_string (state, ">"))
 		return FALSE;
 	
 	return TRUE;
 }
 
+static gboolean
+save_plain (HTMLObject *self,
+	    HTMLEngineSaveState *state,
+	    gint requested_width)
+{
+	HTMLImage *image;
+	gboolean rv = TRUE;
+
+	image = HTML_IMAGE (self);
+	
+	if (image->alt)
+		rv = html_engine_save_output_string (state, "%s", image->alt);
+
+	return rv;
+}
+
 static const gchar *
 get_url (HTMLObject *o)
 {
@@ -457,6 +539,20 @@ get_valign (HTMLObject *self)
 	return image->valign;
 }
 
+static gboolean
+select_range (HTMLObject *self,
+	      HTMLEngine *engine,
+	      guint offset,
+	      gint length,
+	      gboolean queue_draw)
+{
+	if ((*parent_class->select_range) (self, engine, offset, length, queue_draw)) {
+		html_engine_queue_draw (engine, self);
+		return TRUE;
+	} else
+		return FALSE;
+}
+
 
 void
 html_image_type_init (void)
@@ -488,6 +584,8 @@ html_image_class_init (HTMLImageClass *image_class,
 	object_class->accepts_cursor = accepts_cursor;
 	object_class->get_valign = get_valign;
 	object_class->save = save;
+	object_class->save_plain = save_plain;
+	object_class->select_range = select_range;
 
 	parent_class = &html_object_class;
 }
@@ -500,7 +598,8 @@ html_image_init (HTMLImage *image,
 		 const gchar *url,
 		 const gchar *target,
 		 gint16 width, gint16 height,
-		 gint8 percent, gint8 border,
+		 gboolean percent_width, gboolean percent_height,
+		 gint8 border,
 		 HTMLColor *color,
 		 HTMLVAlignType valign)
 {
@@ -515,9 +614,11 @@ html_image_init (HTMLImage *image,
 	image->url = g_strdup (url);
 	image->target = g_strdup (url);
 
-	image->specified_width = width;
+	image->specified_width  = width;
 	image->specified_height = height;
-	image->border = border;
+	image->percent_width    = percent_width;
+	image->percent_height   = percent_height;
+	image->border           = border;
 
 	if (color) {
 		image->color = color;
@@ -529,6 +630,7 @@ html_image_init (HTMLImage *image,
 	}
 
 	image->animation = NULL;
+	image->alt = NULL;
 
 	image->hspace = 0;
 	image->vspace = 0;
@@ -537,8 +639,6 @@ html_image_init (HTMLImage *image,
 		valign = HTML_VALIGN_BOTTOM;
 	image->valign = valign;
 
-	object->percent = percent;
-
 	image->image_ptr = html_image_factory_register (imf, image, filename);
 }
 
@@ -548,7 +648,8 @@ html_image_new (HTMLImageFactory *imf,
 		const gchar *url,
 		const gchar *target,
 		gint16 width, gint16 height,
-		gint8 percent, gint8 border,	
+		gboolean percent_width, gboolean percent_height,
+		gint8 border,	
 		HTMLColor *color,
 		HTMLVAlignType valign)
 {
@@ -562,7 +663,8 @@ html_image_new (HTMLImageFactory *imf,
 			 url,
 			 target,
 			 width, height,
-			 percent, border, 
+			 percent_width, percent_height,
+			 border,
 			 color,
 			 valign);
 
@@ -618,12 +720,26 @@ html_image_set_border (HTMLImage *image, gint border)
 }
 
 void
-html_image_set_size (HTMLImage *image, gint w, gint percent, gint h)
+html_image_set_alt (HTMLImage *image, gchar *alt)
+{
+	if (image->alt)
+		g_free (image->alt);
+
+	image->alt = g_strdup (alt);
+}
+
+void
+html_image_set_size (HTMLImage *image, gint w, gint h, gboolean pw, gboolean ph)
 {
 	gboolean changed = FALSE;
 
-	if (percent != HTML_OBJECT (image)->percent) {
-		HTML_OBJECT (image)->percent = percent;
+	if (pw != image->percent_width) {
+		image->percent_width = pw;
+		changed = TRUE;
+	}
+
+	if (ph != image->percent_height) {
+		image->percent_height = ph;
 		changed = TRUE;
 	}
 
@@ -684,7 +800,7 @@ html_image_factory_area_prepared (GdkPixbufLoader *loader, HTMLImagePointer *ip)
 		while (cur) {
 			if (cur->data) {
 				o = HTML_OBJECT (cur->data);
-				html_object_change_set (o, HTML_CHANGE_MIN_WIDTH);
+				html_object_change_set (o, HTML_CHANGE_ALL);
 			}
 			cur = cur->next;
 		}
