@@ -21,18 +21,22 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include <string.h>
 
-#include "htmlobject.h"
 #include "htmlclue.h"
-#include "htmltext.h"
-#include "htmltextmaster.h"
 #include "htmlclueflow.h"
 #include "htmlcluev.h"
-#include "htmlpainter.h"
-#include "htmlrule.h"
-#include "htmlclue.h"
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
 #include "htmlcursor.h"
+#include "htmlengine.h"
+#include "htmlobject.h"
+#include "htmlpainter.h"
+#include "htmltext.h"
+#include "htmltextmaster.h"
+#include "htmlrule.h"
+#include "htmltype.h"
 
 #include "gtkhtmldebug.h"
 
@@ -58,8 +62,8 @@ destroy (HTMLObject *self)
 		self->next = NULL;
 		self->prev = NULL;
 #endif
+		g_datalist_clear (&self->object_data);
 		g_free (self);
-
 	}
 }
 
@@ -84,6 +88,9 @@ copy (HTMLObject *self,
 	dest->selected = self->selected;
 	dest->free_pending = FALSE;
 	dest->change = self->change;
+
+	g_datalist_init (&dest->object_data);
+	html_object_copy_data_from_object (dest, self);
 }
 
 static void
@@ -131,7 +138,7 @@ fit_line (HTMLObject *o,
 	  gint width_left)
 {
 	html_object_calc_size (o, painter);
-	return (o->width < width_left || first_run) ? HTML_FIT_COMPLETE : HTML_FIT_NONE;
+	return (o->width <= width_left || first_run) ? HTML_FIT_COMPLETE : HTML_FIT_NONE;
 }
 
 static gboolean
@@ -224,6 +231,16 @@ set_bg_color (HTMLObject *o,
 {
 }
 
+static GdkColor *
+get_bg_color (HTMLObject *o,
+	      HTMLPainter *p)
+{
+	
+	return o->parent
+		? html_object_get_bg_color (o->parent, p)
+		: &((html_colorset_get_color (p->color_set, HTMLBgColor))->color);
+}
+
 static HTMLObject*
 check_point (HTMLObject *self,
 	     HTMLPainter *painter,
@@ -382,21 +399,11 @@ select_range (HTMLObject *self,
 	gboolean selected;
 	gboolean changed;
 
-	if (length > 0 || (length == -1 && start < html_object_get_length (self)))
-		selected = TRUE;
-	else
-		selected = FALSE;
-
-	if ((! selected && self->selected) || (selected && ! self->selected))
-		changed = TRUE;
-	else
-		changed = FALSE;
+	selected = length > 0 || (length == -1 && start < html_object_get_length (self)) ? TRUE : FALSE;
+	changed  = (! selected && self->selected) || (selected && ! self->selected) ? TRUE : FALSE;
 
 	self->selected = selected;
 
-	if (queue_draw && changed)
-		html_engine_queue_draw (engine, self);
-
 	return changed;
 }
 
@@ -429,10 +436,18 @@ append_selection_string (HTMLObject *self,
 
 static void
 forall (HTMLObject *self,
+	HTMLEngine *e,
 	HTMLObjectForallFunc func,
 	gpointer data)
 {
-	(* func) (self, data);
+	(* func) (self, e, data);
+}
+
+static HTMLEngine *
+get_engine (HTMLObject *self,
+	    HTMLEngine *e)
+{
+	return e;
 }
 
 static gboolean
@@ -473,13 +488,13 @@ search (HTMLObject *self, HTMLSearch *info)
 static HTMLObject *
 next (HTMLObject *self, HTMLObject *child)
 {
-	return NULL;
+	return child->next;
 }
 
 static HTMLObject *
 prev (HTMLObject *self, HTMLObject *child)
 {
-	return NULL;
+	return child->prev;
 }
 
 static HTMLObject *
@@ -536,6 +551,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->find_anchor = find_anchor;
 	klass->set_link = NULL;
 	klass->set_bg_color = set_bg_color;
+	klass->get_bg_color = get_bg_color;
 	klass->check_point = check_point;
 	klass->relayout = relayout;
 	klass->get_valign = get_valign;
@@ -557,6 +573,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->prev = prev;
 	klass->head = head;
 	klass->tail = tail;
+	klass->get_engine = get_engine;
 }
 
 void
@@ -587,6 +604,8 @@ html_object_init (HTMLObject *o,
 	o->redraw_pending = FALSE;
 	o->free_pending = FALSE;
 	o->selected = FALSE;
+
+	g_datalist_init (&o->object_data);
 }
 
 HTMLObject *
@@ -757,7 +776,7 @@ struct _hlpSetPainter {
 typedef struct _hlpSetPainter hlpSetPainter;
 
 static void
-set_painter_forall (HTMLObject *o, gpointer data)
+set_painter_forall (HTMLObject *o, HTMLEngine *e, gpointer data)
 {
 	hlpSetPainter *sp;
 
@@ -773,7 +792,7 @@ html_object_set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
 	sp = g_new (hlpSetPainter, 1);
 	sp->painter = painter;
 	sp->max_width = max_width;
-	html_object_forall (o, set_painter_forall, sp);
+	html_object_forall (o, NULL, set_painter_forall, sp);
 	g_free (sp);
 }
 
@@ -798,7 +817,11 @@ gint
 html_object_calc_preferred_width (HTMLObject *o,
 				  HTMLPainter *painter)
 {
-	return (* HO_CLASS (o)->calc_preferred_width) (o, painter);
+	if (o->change & HTML_CHANGE_PREF_WIDTH) {
+		o->pref_width = (* HO_CLASS (o)->calc_preferred_width) (o, painter);
+		o->change &= ~HTML_CHANGE_PREF_WIDTH;
+	}
+	return o->pref_width;
 }
 
 const gchar *
@@ -827,6 +850,12 @@ html_object_set_bg_color (HTMLObject *o, GdkColor *color)
 	(* HO_CLASS (o)->set_bg_color) (o, color);
 }
 
+GdkColor *
+html_object_get_bg_color (HTMLObject *o, HTMLPainter *p)
+{
+	return (* HO_CLASS (o)->get_bg_color) (o, p);
+}
+
 HTMLObject *
 html_object_check_point (HTMLObject *self,
 			 HTMLPainter *painter,
@@ -912,12 +941,19 @@ html_object_append_selection_string (HTMLObject *self,
 }
 
 
+HTMLEngine *
+html_object_get_engine (HTMLObject *self, HTMLEngine *e)
+{
+	return (* HO_CLASS (self)->get_engine) (self, e);
+}
+
 void
 html_object_forall (HTMLObject *self,
+		    HTMLEngine *e,
 		    HTMLObjectForallFunc func,
 		    gpointer data)
 {
-	(* HO_CLASS (self)->forall) (self, func, data);
+	(* HO_CLASS (self)->forall) (self, e, func, data);
 }
 
 /* Ugly.  We should have an `is_a' implementation.  */
@@ -951,10 +987,11 @@ html_object_next_not_slave (HTMLObject *object)
 	HTMLObject *p;
 
 	g_return_val_if_fail (object != NULL, NULL);
+	g_return_val_if_fail (object->parent, NULL);
 
-	p = object->next;
-	while (p != NULL && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE)
-		p = p->next;
+	p = html_object_next (object->parent, object);
+	while (p && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE)
+		p = html_object_next (p->parent, p);
 
 	return p;
 }
@@ -965,10 +1002,11 @@ html_object_prev_not_slave (HTMLObject *object)
 	HTMLObject *p;
 
 	g_return_val_if_fail (object != NULL, NULL);
+	g_return_val_if_fail (object->parent, NULL);
 
-	p = object->prev;
-	while (p != NULL && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE)
-		p = p->prev;
+	p = html_object_prev (object->parent, object);
+	while (p && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE)
+		p = html_object_prev (p->parent, p);
 
 	return p;
 }
@@ -1014,7 +1052,7 @@ html_object_change_set (HTMLObject *self, HTMLChangeFlags f)
 }
 
 static void
-change (HTMLObject *o, gpointer data)
+change (HTMLObject *o, HTMLEngine *e, gpointer data)
 {
 	o->change |= GPOINTER_TO_INT (data);
 }
@@ -1022,7 +1060,7 @@ change (HTMLObject *o, gpointer data)
 void
 html_object_change_set_down (HTMLObject *self, HTMLChangeFlags f)
 {
-	html_object_forall (self, (HTMLObjectForallFunc) change, GINT_TO_POINTER (f));
+	html_object_forall (self, NULL, (HTMLObjectForallFunc) change, GINT_TO_POINTER (f));
 }
 
 gboolean
@@ -1183,18 +1221,36 @@ move_object (HTMLObject *obj, HTMLObject * (*next_fn ) (HTMLObject *, HTMLObject
 }
 
 
-HTMLObject *
+inline HTMLObject *
 html_object_next_leaf (HTMLObject *self)
 {
 	return move_object (self, html_object_next, html_object_head);
 }
 
 HTMLObject *
+html_object_next_leaf_not_type (HTMLObject *self, HTMLType t)
+{
+	HTMLObject *rv = self;
+	while ((rv = html_object_next_leaf (rv)) && HTML_OBJECT_TYPE (rv) == t);
+
+	return rv;
+}
+
+inline HTMLObject *
 html_object_prev_leaf (HTMLObject *self)
 {
 	return move_object (self, html_object_prev, html_object_tail);
 }
 
+HTMLObject *
+html_object_prev_leaf_not_type (HTMLObject *self, HTMLType t)
+{
+	HTMLObject *rv = self;
+	while ((rv = html_object_prev_leaf (rv)) && HTML_OBJECT_TYPE (rv) == t);
+
+	return rv;
+}
+
 guint
 html_object_get_bytes (HTMLObject *self)
 {
@@ -1206,3 +1262,29 @@ html_object_get_index (HTMLObject *self, guint offset)
 {
 	return html_object_is_text (self) ? html_text_get_index (HTML_TEXT (self), offset) : offset;
 }
+
+void
+html_object_set_data (HTMLObject *object, const gchar *key, gpointer data)
+{
+	g_datalist_set_data (&object->object_data, key, data);
+}
+
+gpointer
+html_object_get_data (HTMLObject *object, const gchar *key)
+{
+	return g_datalist_get_data (&object->object_data, key);
+}
+
+static void
+copy_data (GQuark key_id, gpointer data, gpointer user_data)
+{
+	HTMLObject *o = HTML_OBJECT (user_data);
+
+	g_datalist_id_set_data (&o->object_data, key_id, data);
+}
+
+void
+html_object_copy_data_from_object (HTMLObject *dst, HTMLObject *src)
+{
+	g_datalist_foreach (&src->object_data, copy_data, dst);
+}
