@@ -24,79 +24,16 @@
 #ifndef _HTMLOBJECT_H_
 #define _HTMLOBJECT_H_
 
-#include <glib.h>
-
-typedef struct _HTMLObjectClass HTMLObjectClass;
-typedef struct _HTMLAnchor HTMLAnchor;
-typedef struct _HTMLObject HTMLObject;
-typedef struct _HTMLSearch HTMLSearch;
-
-/* FIXME fix ugly dependency on HTMLForm.  */
-typedef struct _HTMLForm HTMLForm;
-/* FIXME fix ugly dependency on HTMLSelect.  */
-typedef struct _HTMLSelect HTMLSelect;
-/* FIXME fix ugly dependency on HTMLTextArea.  */
-typedef struct _HTMLTextArea HTMLTextArea;
-
-typedef enum {
-	HTML_FIT_NONE,
-	HTML_FIT_PARTIAL,
-	HTML_FIT_COMPLETE
-} HTMLFitType;
-
-enum HTMLObjectFlags {
-	HTML_OBJECT_FLAG_NONE = 0,
-	HTML_OBJECT_FLAG_SEPARATOR = 1 << 0,
-	HTML_OBJECT_FLAG_NEWLINE = 1 << 1,
-	HTML_OBJECT_FLAG_SELECTED = 1 << 2,
-	HTML_OBJECT_FLAG_ALLSELECTED = 1 << 3,
-	HTML_OBJECT_FLAG_FIXEDWIDTH = 1 << 4,
-	HTML_OBJECT_FLAG_ALIGNED = 1 << 5,
-	HTML_OBJECT_FLAG_PRINTED = 1 << 6,
-	HTML_OBJECT_FLAG_HIDDEN = 1 << 7
-};
-
-typedef enum {
-	HTML_CLEAR_NONE,
-	HTML_CLEAR_LEFT,
-	HTML_CLEAR_RIGHT,
-	HTML_CLEAR_ALL
-} HTMLClearType;
-
-typedef enum {
-	HTML_VALIGN_TOP,
-	HTML_VALIGN_CENTER,
-	HTML_VALIGN_BOTTOM,
-	HTML_VALIGN_NONE
-} HTMLVAlignType;
-
-typedef enum {
-	HTML_HALIGN_LEFT,
-	HTML_HALIGN_CENTER,
-	HTML_HALIGN_RIGHT,
-	HTML_HALIGN_NONE
-} HTMLHAlignType;
-
-typedef enum {
-	HTML_CHANGE_NONE = 0,
-	HTML_CHANGE_MIN_WIDTH = 1 << 0,
-	HTML_CHANGE_ALL = 0x1
-} HTMLChangeFlags;
-
-
-#include "htmlengine.h"
-#include "htmlcursor.h"
-#include "htmltype.h"
-#include "htmlpainter.h"
-#include "htmlengine-save.h"
+#include <libart_lgpl/art_rect.h>
+#include <gdk/gdktypes.h>
+#include "htmltypes.h"
+#include "htmlenums.h"
 
 
 #define HTML_OBJECT(x)		((HTMLObject *) (x))
 #define HTML_OBJECT_CLASS(x)	((HTMLObjectClass *) (x))
 #define HTML_OBJECT_TYPE(x)     (HTML_OBJECT (x)->klass->type)
 
-typedef void (* HTMLObjectForallFunc) (HTMLObject *self, gpointer data);
-
 struct _HTMLObject {
 	HTMLObjectClass *klass;
 
@@ -114,6 +51,7 @@ struct _HTMLObject {
 
 	gint min_width;
 	gint width;
+	gint pref_width;
 	gint max_width;
 
 	gint percent;
@@ -128,6 +66,8 @@ struct _HTMLObject {
            is set to TRUE instead of g_free()ing the object.  When the draw
            queue is flushed, the g_free() is performed.  */
 	guint free_pending : 1;
+
+	GData *object_data;
 };
 
 struct _HTMLObjectClass {
@@ -185,6 +125,8 @@ struct _HTMLObjectClass {
 				  gint x, gint y,
 				  gint width, gint height,
 				  gint tx, gint ty);
+	void       (* set_bg_color) (HTMLObject *o, GdkColor *color);
+	GdkColor * (* get_bg_color) (HTMLObject *o, HTMLPainter *p);
 
 	/* Margins.  This should actually be used only by objects that
            contain other objects, so it should be in HTMLClue.  But
@@ -206,8 +148,6 @@ struct _HTMLObjectClass {
 
 	HTMLAnchor * (* find_anchor) (HTMLObject *o, const gchar *name, gint *x, gint *y);
 
-	void (* set_bg_color) (HTMLObject *o, GdkColor *color);
-
 	HTMLObject * (* check_point) (HTMLObject *self, HTMLPainter *painter,
 				      gint x, gint y, guint *offset_return,
 				      gboolean for_cursor);
@@ -235,8 +175,8 @@ struct _HTMLObjectClass {
 				  gint *x, gint *y);
 
 	/* Container operations.  */
-
-	void (* forall) (HTMLObject *self, HTMLObjectForallFunc func, gpointer data);
+	HTMLEngine * (* get_engine) (HTMLObject *o, HTMLEngine *e);
+	void (* forall) (HTMLObject *self, HTMLEngine *e, HTMLObjectForallFunc func, gpointer data);
 	gboolean (* is_container) (HTMLObject *self);
 
 	/* Saving.  */
@@ -300,7 +240,10 @@ void        html_object_set_painter       (HTMLObject           *o,
 					   gint                  max_width);
 void        html_object_reset             (HTMLObject           *o);
 gboolean    html_object_is_text           (HTMLObject           *object);
+HTMLEngine *html_object_get_engine        (HTMLObject           *self,
+					   HTMLEngine           *e);
 void        html_object_forall            (HTMLObject           *self,
+					   HTMLEngine           *e,
 					   HTMLObjectForallFunc  func,
 					   gpointer              data);
 gboolean    html_object_is_container      (HTMLObject           *self);
@@ -334,9 +277,11 @@ void      html_object_draw_background  (HTMLObject  *o,
 					gint         height,
 					gint         tx,
 					gint         ty);
-gboolean  html_object_is_transparent   (HTMLObject  *self);
 void      html_object_set_bg_color     (HTMLObject  *o,
 					GdkColor    *color);
+GdkColor *html_object_get_bg_color     (HTMLObject  *o,
+					HTMLPainter *p);
+gboolean  html_object_is_transparent   (HTMLObject  *self);
 
 /* Layout.  */
 HTMLFitType     html_object_fit_line              (HTMLObject  *o,
@@ -431,8 +376,12 @@ HTMLObject * html_object_head            (HTMLObject *self);
 HTMLObject * html_object_tail            (HTMLObject *self);
 
 /* get prev/next leaf object in scope of whole tree */
-HTMLObject * html_object_next_leaf       (HTMLObject *self);
-HTMLObject * html_object_prev_leaf       (HTMLObject *self);
+HTMLObject *html_object_next_leaf           (HTMLObject *self);
+HTMLObject *html_object_prev_leaf           (HTMLObject *self);
+HTMLObject *html_object_next_leaf_not_type  (HTMLObject *self,
+					     HTMLType    t);
+HTMLObject *html_object_prev_leaf_not_type  (HTMLObject *self,
+					     HTMLType    t);
 
 /* Page splitting.  */
 gint  html_object_check_page_split  (HTMLObject *self,
@@ -465,4 +414,14 @@ void  html_object_change_set  (HTMLObject      *self,
 void  html_object_change_set_down  (HTMLObject      *self,
 				    HTMLChangeFlags  f);
 
+/* object data */
+
+void      html_object_set_data               (HTMLObject  *object,
+					      const gchar *key,
+					      gpointer     data);
+gpointer  html_object_get_data               (HTMLObject  *object,
+					      const gchar *key);
+void      html_object_copy_data_from_object  (HTMLObject  *dst,
+					      HTMLObject  *src);
+
 #endif /* _HTMLOBJECT_H_ */
