@@ -21,6 +21,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include "htmlcolorset.h"
 #include "htmlpainter.h"
 
 
@@ -76,9 +78,7 @@ DEFINE_UNIMPLEMENTED (get_black);
 DEFINE_UNIMPLEMENTED (draw_line);
 DEFINE_UNIMPLEMENTED (draw_rect);
 DEFINE_UNIMPLEMENTED (draw_text);
-#ifdef GTKHTML_HAVE_PSPELL
 DEFINE_UNIMPLEMENTED (draw_spell_error);
-#endif
 DEFINE_UNIMPLEMENTED (fill_rect);
 DEFINE_UNIMPLEMENTED (draw_pixmap);
 DEFINE_UNIMPLEMENTED (draw_ellipse);
@@ -133,9 +133,7 @@ class_init (GtkObjectClass *object_class)
 	class->draw_line = (gpointer) draw_line_unimplemented;
 	class->draw_rect = (gpointer) draw_rect_unimplemented;
 	class->draw_text = (gpointer) draw_text_unimplemented;
-#ifdef GTKHTML_HAVE_PSPELL
 	class->draw_spell_error = (gpointer) draw_spell_error_unimplemented;
-#endif
 	class->fill_rect = (gpointer) fill_rect_unimplemented;
 	class->draw_pixmap = (gpointer) draw_pixmap_unimplemented;
 	class->draw_ellipse = (gpointer) draw_ellipse_unimplemented;
@@ -426,6 +424,7 @@ html_painter_draw_shade_line (HTMLPainter *painter,
 
 void
 html_painter_draw_panel (HTMLPainter *painter,
+			 GdkColor *bg,
 			 gint x, gint y,
 			 gint width, gint height,
 			 GtkHTMLEtchStyle inset,
@@ -434,7 +433,7 @@ html_painter_draw_panel (HTMLPainter *painter,
 	g_return_if_fail (painter != NULL);
 	g_return_if_fail (HTML_IS_PAINTER (painter));
 
-	(* HP_CLASS (painter)->draw_panel) (painter, x, y, width, height, inset, bordersize);
+	(* HP_CLASS (painter)->draw_panel) (painter, bg, x, y, width, height, inset, bordersize);
 }
 
 /* Passing 0 for width/height means remove clip rectangle */
@@ -473,7 +472,6 @@ html_painter_get_pixel_size (HTMLPainter *painter)
 	return (* HP_CLASS (painter)->get_pixel_size) (painter);
 }
 
-#ifdef GTKHTML_HAVE_PSPELL
 void
 html_painter_draw_spell_error (HTMLPainter *painter,
 			       gint x, gint y,
@@ -482,4 +480,3 @@ html_painter_draw_spell_error (HTMLPainter *painter,
 {
 	(* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, off, len);
 }
-#endif
