@@ -24,14 +24,12 @@
 #ifndef _HTMLPAINTER_H_
 #define _HTMLPAINTER_H_
 
-typedef struct _HTMLPainter HTMLPainter;
-
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtk.h>
+#include <gtk/gtkobject.h>
 
-#include "gtkhtmlfontstyle.h"
+#include "gtkhtml-enums.h"
+#include "htmltypes.h"
 #include "htmlfontmanager.h"
-#include "htmlcolorset.h"
 
 
 #define HTML_TYPE_PAINTER                 (html_painter_get_type ())
@@ -40,13 +38,6 @@ typedef struct _HTMLPainter HTMLPainter;
 #define HTML_IS_PAINTER(obj)              (GTK_CHECK_TYPE ((obj), HTML_TYPE_PAINTER))
 #define HTML_IS_PAINTER_CLASS(klass)      (GTK_CHECK_CLASS_TYPE ((klass), HTML_TYPE_PAINTER))
 
-
-typedef enum {
-	GTK_HTML_ETCH_IN,
-	GTK_HTML_ETCH_OUT,
-	GTK_HTML_ETCH_NONE
-} GtkHTMLEtchStyle; 
-
 
 struct _HTMLPainter {
 	GtkObject base;
@@ -80,9 +71,9 @@ struct _HTMLPainterClass {
 	void (* draw_line) (HTMLPainter *painter, gint x1, gint y1, gint x2, gint y2);
 	void (* draw_rect) (HTMLPainter *painter, gint x, gint y, gint width, gint height);
 	void (* draw_text) (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len);
-#ifdef GTKHTML_HAVE_PSPELL
+
 	void (* draw_spell_error) (HTMLPainter *painter, gint x, gint y, const gchar *text, guint off, gint len);
-#endif
+
 	void (* fill_rect) (HTMLPainter *painter, gint x, gint y, gint width, gint height);
 	void (* draw_pixmap) (HTMLPainter *painter, GdkPixbuf *pixbuf, gint x, gint y,
 			      gint scale_width, gint scale_height, const GdkColor *color);
@@ -90,7 +81,7 @@ struct _HTMLPainterClass {
 	void (* clear) (HTMLPainter *painter);
 	void (* set_background_color) (HTMLPainter *painter, const GdkColor *color);
 	void (* draw_shade_line) (HTMLPainter *p, gint x, gint y, gint width);
-	void (* draw_panel) (HTMLPainter *painter,
+	void (* draw_panel) (HTMLPainter *painter, GdkColor *bg,
 			     gint x, gint y, gint width, gint height,
 			     GtkHTMLEtchStyle inset, gint bordersize);
 
@@ -100,7 +91,6 @@ struct _HTMLPainterClass {
 	
 	guint (* get_pixel_size) (HTMLPainter *painter);
 };
-typedef struct _HTMLPainterClass HTMLPainterClass;
 
 
 /* Creation.  */
@@ -197,6 +187,7 @@ void  html_painter_draw_shade_line       (HTMLPainter    *p,
 					  gint            y,
 					  gint            width);
 void  html_painter_draw_panel            (HTMLPainter    *painter,
+					  GdkColor       *bg,
 					  gint            x,
 					  gint            y,
 					  gint            width,
@@ -224,11 +215,9 @@ void  html_painter_draw_background  (HTMLPainter *painter,
 
 guint  html_painter_get_pixel_size  (HTMLPainter *painter);
 
-#ifdef GTKHTML_HAVE_PSPELL
 void   html_painter_draw_spell_error (HTMLPainter *painter,
 				      gint x, gint y,
 				      const gchar *text,
 				      guint off, gint len);
-#endif
 
 #endif /* _HTMLPAINTER_H_ */
