@@ -21,7 +21,12 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
+#include "htmlengine-save.h"
 #include "htmlrule.h"
+#include "htmlpainter.h"
 
 
 HTMLRuleClass html_rule_class;
@@ -38,7 +43,9 @@ copy (HTMLObject *self,
 {
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
 
-	HTML_RULE (dest)->shade = HTML_RULE (self)->shade;
+	HTML_RULE (dest)->length = HTML_RULE (self)->length;
+	HTML_RULE (dest)->size   = HTML_RULE (self)->size;
+	HTML_RULE (dest)->shade  = HTML_RULE (self)->shade;
 	HTML_RULE (dest)->halign = HTML_RULE (self)->halign;
 }
 
@@ -62,6 +69,27 @@ calc_min_width (HTMLObject *o,
 	return pixel_size;
 }
 
+static HTMLFitType
+fit_line (HTMLObject *o,
+	  HTMLPainter *painter,
+	  gboolean start_of_line,
+	  gboolean first_run,
+	  gint width_left)
+{
+	if (!start_of_line)
+		return HTML_FIT_NONE;
+	o->width = width_left;
+
+	if (o->percent == 0) {
+		gint pixel_size = html_painter_get_pixel_size (painter);
+		if (HTML_RULE (o)->length * pixel_size > width_left) {
+			o->width = HTML_RULE (o)->length * pixel_size;
+		}
+	}
+
+	return HTML_FIT_COMPLETE;
+}
+
 static gboolean
 calc_size (HTMLObject *self,
 	   HTMLPainter *painter)
@@ -86,7 +114,7 @@ calc_size (HTMLObject *self,
 
 	changed = FALSE;
 
-	if (self->width != self->max_width) {
+	if (self->width > self->max_width) {
 		self->width = self->max_width;
 		changed = TRUE;
 	}
@@ -125,16 +153,12 @@ draw (HTMLObject *o,
 	xp = o->x + tx;
 	yp = o->y + ty - (rule->size / 2 + rule->size % 2) * pixel_size;
 
-	if (o->percent == 0) {
-		if (HTML_RULE (o)->length > 0)
-			w = HTML_RULE (o)->length * pixel_size;
-		else
-			w = o->max_width;
-	} else {
+	if (o->percent == 0)
+		w = o->width;
+	else
 		/* The cast to `gdouble' is to avoid overflow (eg. when
                    printing).  */
-		w = ((gdouble) o->max_width * o->percent) / 100;
-	}
+		w = ((gdouble) o->width * o->percent) / 100;
 
 	switch (rule->halign) {
 	case HTML_HALIGN_LEFT:
@@ -152,7 +176,8 @@ draw (HTMLObject *o,
 	}
 
 	if (rule->shade)
-		html_painter_draw_panel (p, xp, yp, w, h, GTK_HTML_ETCH_IN, 1);
+		html_painter_draw_panel (p, &((html_colorset_get_color (p->color_set, HTMLBgColor))->color),
+					 xp, yp, w, h, GTK_HTML_ETCH_IN, 1);
 	else {
 		html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLTextColor)->color);
 		html_painter_fill_rect (p, xp, yp, w, h);
@@ -173,6 +198,26 @@ save (HTMLObject *self,
 	return html_engine_save_output_string (state, "\n<HR%s>\n", HTML_RULE (self)->shade ? "" : " noshade");
 }
 
+static gboolean
+save_plain (HTMLObject *self,
+	    HTMLEngineSaveState *state,
+	    gint requested_width)
+{
+	int i;
+	
+	if (!html_engine_save_output_string (state, "\n"))
+		return FALSE;
+
+	/* Fixme no alignment or percent */
+	for (i = 0; i < requested_width; i++)
+		if (!html_engine_save_output_string (state, "_"))
+			return FALSE;
+	
+	if (!html_engine_save_output_string (state, "\n"))
+		return FALSE;
+
+	return TRUE;
+}
 
 void
 html_rule_type_init (void)
@@ -195,9 +240,11 @@ html_rule_class_init (HTMLRuleClass *klass,
 	object_class->draw = draw;
 	object_class->set_max_width = set_max_width;
 	object_class->calc_min_width = calc_min_width;
+	object_class->fit_line = fit_line;
 	object_class->calc_size = calc_size;
 	object_class->accepts_cursor = accepts_cursor;
 	object_class->save = save;
+	object_class->save_plain = save_plain;
 
 	parent_class = &html_object_class;
 }
