@@ -21,11 +21,14 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include <glib.h>
-#include "htmlcluev.h"
-#include "htmltablecell.h"
-#include "htmlobject.h"
 #include <libart_lgpl/art_rect.h>
+#include "htmlcluev.h"
+#include "htmlengine-save.h"
+#include "htmlpainter.h"
+#include "htmltable.h"
+#include "htmltablecell.h"
 
 /* FIXME: This always behaves as a transparent object, even when it
    actually is not.  */
@@ -127,6 +130,15 @@ calc_min_width (HTMLObject *o,
 	return minWidth;
 }
 
+static gint
+calc_preferred_width (HTMLObject *o,
+		      HTMLPainter *painter)
+{
+	return o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH
+		? html_object_calc_min_width (o, painter)
+		: (* HTML_OBJECT_CLASS (parent_class)->calc_preferred_width) (o, painter);
+}
+
 static void
 draw (HTMLObject *o,
       HTMLPainter *p,
@@ -184,6 +196,45 @@ set_bg_color (HTMLObject *object, GdkColor *color)
 	cell->have_bg = TRUE;
 }
 
+static GdkColor *
+get_bg_color (HTMLObject *o,
+	      HTMLPainter *p)
+{
+	
+	return HTML_TABLE_CELL (o)->have_bg
+		? &HTML_TABLE_CELL (o)->bg
+		: html_object_get_bg_color (o->parent, p);
+}
+
+
+#define SB if (!html_engine_save_output_string (state,
+#define SE )) return FALSE
+
+static gboolean
+save (HTMLObject *self,
+      HTMLEngineSaveState *state)
+{
+	HTMLTableCell *cell = HTML_TABLE_CELL (self);
+
+	SB "<TD" SE;
+	if (cell->have_bg
+	    && (!HTML_TABLE (self->parent)->bgColor || !gdk_color_equal (&cell->bg, HTML_TABLE (self->parent)->bgColor)))
+		SB " BGCOLOR=\"#%02x%02x%02x\"",
+			cell->bg.red >> 8,
+			cell->bg.green >> 8,
+			cell->bg.blue >> 8 SE;
+	if (self->percent > 0) {
+		SB " WIDTH=\"%d%%\"", self->percent SE;
+	} else if (self->flags & HTML_OBJECT_FLAG_FIXEDWIDTH)
+		SB " WIDTH=\"%d\"", cell->fixed_width SE;
+	SB ">\n" SE;
+	if (!(*HTML_OBJECT_CLASS (parent_class)->save) (self, state))
+		return FALSE;
+	SB "</TD>\n" SE;
+
+	return TRUE;
+}
+
 
 void
 html_table_cell_type_init (void)
@@ -207,9 +258,12 @@ html_table_cell_class_init (HTMLTableCellClass *klass,
 	object_class->reset = reset;
 	object_class->copy = copy;
 	object_class->calc_min_width = calc_min_width;
+	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->draw = draw;
 	object_class->draw_background = draw_background;
 	object_class->set_bg_color = set_bg_color;
+	object_class->get_bg_color = get_bg_color;
+	object_class->save = save;
 
 	parent_class = &html_cluev_class;
 }
