@@ -21,14 +21,19 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
-#include <unicode.h>
 
 #include "htmltext.h"
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
 #include "htmlclueflow.h"
-#include "htmlcursor.h"
+#include "htmlengine.h"
+#include "htmlengine-save.h"
 #include "htmlentity.h"
+#include "htmlsettings.h"
+#include "htmltextslave.h"
 
 
 HTMLTextClass html_text_class;
@@ -36,12 +41,10 @@ static HTMLObjectClass *parent_class = NULL;
 
 #define HT_CLASS(x) HTML_TEXT_CLASS (HTML_OBJECT (x)->klass)
 
-#ifdef GTKHTML_HAVE_PSPELL
 static SpellError * spell_error_new     (guint off, guint len);
 static void         spell_error_destroy (SpellError *se);
 static void         move_spell_errors   (GList *spell_errors, guint offset, gint delta);
 static GList *      remove_spell_errors (GList *spell_errors, guint offset, guint len);
-#endif
 
 
 static void
@@ -66,57 +69,57 @@ get_tags (const HTMLText *text,
 
 	if (!html_color_equal (text->color, html_colorset_get_color (state->engine->settings->color_set, HTMLTextColor))
 	    && !html_color_equal (text->color, html_colorset_get_color (state->engine->settings->color_set, HTMLLinkColor))) {
-		opening_p += sprintf (opening_p, "<font color=#%02x%02x%02x>",
+		opening_p += sprintf (opening_p, "<FONT COLOR=#%02x%02x%02x>",
 				      text->color->color.red   >> 8,
 				      text->color->color.green >> 8,
 				      text->color->color.blue  >> 8);
-		ending_p += sprintf (ending_p, "</font>");
+		ending_p += sprintf (ending_p, "</FONT>");
 	}
 
 	size = font_style & GTK_HTML_FONT_STYLE_SIZE_MASK;
 	if (size != 0) {
-		opening_p += sprintf (opening_p, "<font size=%d>", size);
+		opening_p += sprintf (opening_p, "<FONT SIZE=%d>", size);
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_BOLD) {
-		opening_p += sprintf (opening_p, "<b>");
+		opening_p += sprintf (opening_p, "<B>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_ITALIC) {
-		opening_p += sprintf (opening_p, "<i>");
+		opening_p += sprintf (opening_p, "<I>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
-		opening_p += sprintf (opening_p, "<u>");
+		opening_p += sprintf (opening_p, "<U>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
-		opening_p += sprintf (opening_p, "<s>");
+		opening_p += sprintf (opening_p, "<S>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_FIXED) {
-		opening_p += sprintf (opening_p, "<tt>");
-		ending_p += sprintf (ending_p, "</tt>");
+		opening_p += sprintf (opening_p, "<TT>");
+		ending_p += sprintf (ending_p, "</TT>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
-		ending_p += sprintf (ending_p, "</s>");
+		ending_p += sprintf (ending_p, "</S>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_UNDERLINE) {
-		ending_p += sprintf (ending_p, "</u>");
+		ending_p += sprintf (ending_p, "</U>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_ITALIC) {
-		ending_p += sprintf (ending_p, "</i>");
+		ending_p += sprintf (ending_p, "</I>");
 	}
 
 	if (font_style & GTK_HTML_FONT_STYLE_BOLD) {
-		ending_p += sprintf (ending_p, "</b>");
+		ending_p += sprintf (ending_p, "</B>");
 	}
 
 	if (size != 0) {
-		ending_p += sprintf (ending_p, "</font size=%d>", size);
+		ending_p += sprintf (ending_p, "</FONT SIZE=%d>", size);
 	}
 
 
@@ -145,7 +148,6 @@ copy_helper (HTMLText *src,
 	dest->color = src->color;
 	html_color_ref (dest->color);
 
-#ifdef GTKHTML_HAVE_PSPELL
 	dest->spell_errors = g_list_copy (src->spell_errors);
 	cur = dest->spell_errors;
 	while (cur) {
@@ -156,7 +158,6 @@ copy_helper (HTMLText *src,
 	dest->spell_errors = remove_spell_errors (dest->spell_errors, 0, offset);
 	dest->spell_errors = remove_spell_errors (dest->spell_errors, offset + len, src->text_len - offset - len);
 	move_spell_errors (dest->spell_errors, 0, -offset);
-#endif
 }
 
 
@@ -370,39 +371,17 @@ save (HTMLObject *self,
 	return TRUE;
 }
 
-static guchar *
-encode_text_to_8bit (gchar *utf8)
-{
-	guchar *raw;
-	guint i, len;
-	unicode_char_t uc;
-
-	len = unicode_strlen (utf8, -1);
-	raw = g_new (gchar, len + 1);
-	raw [len] = 0;
-
-	for (i = 0; i < len; i++) {
-		utf8 = unicode_get_utf8 (utf8, &uc);
-		/* raw [i] = uc == ENTITY_NBSP ? ' ' : uc & 0xff; */
-		raw [i] = uc & 0xff;
-	}
-
-	return raw;
-}
-
 static gboolean
 save_plain (HTMLObject *self,
 	    HTMLEngineSaveState *state,
 	    gint requested_width)
 {
 	HTMLText *text;
-	guchar *raw;
 	gboolean rv;
+
 	text = HTML_TEXT (self);
 
-	raw = encode_text_to_8bit (text->text);
-	rv  = html_engine_save_output_string (state, "%s", raw);
-	g_free (raw);
+	rv  = html_engine_save_output_string (state, "%s", text->text);
 	
 	return rv;
 }
@@ -514,7 +493,6 @@ convert_nbsp (gchar *s)
 	return rv;
 }
 
-#ifdef GTKHTML_HAVE_PSPELL
 static void 
 move_spell_errors (GList *spell_errors, guint offset, gint delta) 
 { 
@@ -548,7 +526,6 @@ remove_spell_errors (GList *spell_errors, guint offset, guint len)
   	} 
 	return spell_errors;
 }
-#endif
 
 static guint
 insert_text (HTMLText *text,
@@ -583,10 +560,9 @@ insert_text (HTMLText *text,
 	memcpy (new_buffer + l1 + ls, text->text + l1, l2);
 	new_buffer [l1 + l2 + ls] = '\0';
 
-#ifdef GTKHTML_HAVE_PSPELL
 	/* spell checking update */
 	move_spell_errors (text->spell_errors, offset, len);
-#endif
+
 	/* set new values */
 	g_free (text->text);
 	text->text = convert_nbsp (new_buffer);
@@ -642,11 +618,10 @@ remove_text (HTMLText *text,
 	memcpy (new_buffer,      text->text,      l1);
 	memcpy (new_buffer + l1, text->text + l2, lw - l2 + 1);
 
-#ifdef GTKHTML_HAVE_PSPELL
 	/* spell checking update */
 	text->spell_errors = remove_spell_errors (text->spell_errors, offset, len);
 	move_spell_errors (text->spell_errors, offset + len, -len);
-#endif
+
 	/* set new values */
 	g_free (text->text);
 	text->text = convert_nbsp (new_buffer);
@@ -695,7 +670,6 @@ get_cursor_base (HTMLObject *self,
 
 
 
-#ifdef GTKHTML_HAVE_PSPELL
 static void
 split_spell_errors (HTMLText *self, HTMLText *new, guint offset)
 {
@@ -717,7 +691,6 @@ split_spell_errors (HTMLText *self, HTMLText *new, guint offset)
 		cur = cur->next;
 	}
 }
-#endif
 
 static HTMLText *
 split (HTMLText *self,
@@ -732,11 +705,8 @@ split (HTMLText *self,
 	self->text = g_realloc (self->text, unicode_offset_to_index (self->text, offset) + 1);
 	self->text [unicode_offset_to_index (self->text, offset)] = '\0';
 	self->text_len = offset;
-	html_object_change_set (HTML_OBJECT (self), HTML_CHANGE_MIN_WIDTH);
-
-#ifdef GTKHTML_HAVE_PSPELL
+	html_object_change_set (HTML_OBJECT (self), HTML_CHANGE_ALL);
 	split_spell_errors (self, new, offset);
-#endif
 
 	return new;
 }
@@ -835,13 +805,36 @@ destroy (HTMLObject *obj)
 {
 	HTMLText *text = HTML_TEXT (obj);
 	html_color_unref (text->color);
-#ifdef GTKHTML_HAVE_PSPELL
 	html_text_spell_errors_clear (text);
-#endif
 	g_free (text->text);
 	HTML_OBJECT_CLASS (parent_class)->destroy (obj);
 }
 
+static gboolean
+select_range (HTMLObject *self,
+	      HTMLEngine *engine,
+	      guint start,
+	      gint length,
+	      gboolean queue_draw)
+{
+	if (queue_draw && length) {
+		HTMLObject *o = self->next;
+
+		while (o && HTML_OBJECT_TYPE (o) == HTML_TYPE_TEXTSLAVE) {
+			HTMLTextSlave *slave = HTML_TEXT_SLAVE (o);
+			g_assert (HTML_OBJECT (slave->owner) == self);
+			if (MAX (slave->posStart, start) < MIN (slave->posStart + slave->posLen, start + length)
+			    && queue_draw) {
+				
+				html_engine_queue_draw (engine, o);
+			}
+			o = o->next;
+		}
+	}
+
+	return HTML_OBJECT_CLASS (parent_class)->select_range (self, engine, start, length, queue_draw);
+}
+
 
 void
 html_text_type_init (void)
@@ -871,6 +864,7 @@ html_text_class_init (HTMLTextClass *klass,
 	object_class->get_cursor_base = get_cursor_base;
 	object_class->save = save;
 	object_class->save_plain = save_plain;
+	object_class->select_range = select_range;
 	object_class->get_length = get_length;
 
 	/* HTMLText methods.  */
@@ -914,13 +908,11 @@ html_text_init (HTMLText *text_object,
 		text_object->text = g_strndup (text, unicode_offset_to_index (text, len));
 	}
 
-	text_object->font_style = font_style;
-	text_object->face = NULL;
-	html_color_ref (color);
-	text_object->color = color;
-#ifdef GTKHTML_HAVE_PSPELL
+	text_object->font_style   = font_style;
+	text_object->face         = NULL;
+	text_object->color        = color;
 	text_object->spell_errors = NULL;
-#endif
+	html_color_ref (color);
 }
 
 HTMLObject *
@@ -1086,8 +1078,6 @@ html_text_set_text (HTMLText *text, const gchar *new_text)
 	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
 }
 
-#ifdef GTKHTML_HAVE_PSPELL
-
 /* spell checking */
 
 #include "htmlinterval.h"
@@ -1173,8 +1163,6 @@ html_text_spell_errors_add (HTMLText *text, guint off, guint len)
 	printf ("---------------------------------------\n"); */
 }
 
-#endif
-
 guint
 html_text_get_bytes (HTMLText *text)
 {
