@@ -25,12 +25,6 @@
 
 #include <unicode.h>
 #include "htmlobject.h"
-#include "htmlfontmanager.h"
-#include "htmlcolor.h"
-#include "htmlinterval.h"
-
-typedef struct _HTMLText HTMLText;
-typedef struct _HTMLTextClass HTMLTextClass;
 
 #define HTML_TEXT(x) ((HTMLText *)(x))
 #define HTML_TEXT_CLASS(x) ((HTMLTextClass *)(x))
@@ -45,9 +39,7 @@ struct _HTMLText {
 	HTMLFontFace     *face;
 	HTMLColor        *color;
 
-#ifdef GTKHTML_HAVE_PSPELL
 	GList *spell_errors;
-#endif
 };
 
 struct _HTMLTextClass {
@@ -140,13 +132,10 @@ unicode_char_t    html_text_get_char        (HTMLText         *text,
 gchar            *html_text_get_text        (HTMLText         *text,
 					     guint             offset);
 
-#ifdef GTKHTML_HAVE_PSPELL
-
 struct _SpellError {
 	guint off;
 	guint len;
 };
-typedef struct _SpellError SpellError;
 
 void  html_text_spell_errors_clear           (HTMLText     *text);
 void  html_text_spell_errors_clear_interval  (HTMLText     *text,
@@ -154,6 +143,4 @@ void  html_text_spell_errors_clear_interval  (HTMLText     *text,
 void  html_text_spell_errors_add             (HTMLText     *text,
 					      guint         off,
 					      guint         len);
-#endif
-
 #endif /* _HTMLTEXT_H_ */
