@@ -21,19 +21,23 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include "htmlobject.h"
+#include <config.h>
+#include <string.h>
+#include <sys/types.h>
+#include <regex.h>
+
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
 #include "htmlclue.h"
 #include "htmlclueflow.h"
 #include "htmltextmaster.h"
 #include "htmltextslave.h"
 #include "htmlentity.h"
-#include <string.h>
-#include <sys/types.h>
-#include <regex.h>
 #include "htmlengine.h"
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-paste.h"
 #include "htmllinktextmaster.h"
+#include "htmlsettings.h"
 
 
 HTMLTextMasterClass html_text_master_class;
@@ -75,9 +79,12 @@ static gboolean
 calc_size (HTMLObject *self,
 	   HTMLPainter *painter)
 {
+	HTMLText *text = HTML_TEXT (self);
+	GtkHTMLFontStyle style = html_text_get_font_style (text);
+
 	self->width = 0;
-	self->ascent = 0;
-	self->descent = 0;
+	self->ascent = html_painter_calc_ascent (painter, style, text->face);
+	self->descent = html_painter_calc_descent (painter, style, text->face);
 
 	return FALSE;
 }
@@ -130,7 +137,7 @@ check_point (HTMLObject *self,
 	     p != NULL && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE;
 	     p = p->next) {
 
-		if (y > p->y + p->descent)
+		if (y >= p->y + p->descent)
 			continue;
 
 		/* If the cursor is on this line, there is a newline
