@@ -21,11 +21,15 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
 #include <string.h>
 
 #include "htmltextslave.h"
 #include "htmlclue.h"
 #include "htmlcursor.h"
+#include "htmlcolor.h"
+#include "htmlcolorset.h"
+#include "htmlpainter.h"
 
 
 /* #define HTML_TEXT_SLAVE_DEBUG */
@@ -86,6 +90,19 @@ calc_size (HTMLObject *self,
 	new_ascent = html_painter_calc_ascent (painter, font_style, owner->face);
 	new_descent = html_painter_calc_descent (painter, font_style, owner->face);
 
+	/* handle sub & super script */
+	if (font_style & GTK_HTML_FONT_STYLE_SUBSCRIPT || font_style & GTK_HTML_FONT_STYLE_SUPERSCRIPT) {
+		gint shift = (new_ascent + new_descent) >> 1;
+
+		if (font_style & GTK_HTML_FONT_STYLE_SUBSCRIPT) {
+			new_descent += shift;
+			new_ascent  -= shift;
+		} else {
+			new_descent -= shift;
+			new_ascent  += shift;
+		}
+	}
+
 	new_width = html_painter_calc_text_width (painter,
 						  html_text_get_text (HTML_TEXT (owner), slave->posStart),
 						  slave->posLen,
@@ -155,6 +172,23 @@ get_next_nb_width (HTMLTextSlave *slave, HTMLPainter *painter)
 	return width;
 }
 
+static gboolean
+could_remove_leading_space (HTMLTextSlave *slave, gboolean firstRun)
+{
+	HTMLObject *o = HTML_OBJECT (slave->owner);
+
+	if (firstRun && (HTML_OBJECT (slave)->prev != o || o->prev))
+		return TRUE;
+
+	if (!o->prev)
+		return FALSE;
+
+	while (o->prev && HTML_OBJECT_TYPE (o->prev) == HTML_TYPE_CLUEALIGNED)
+		o = o->prev;
+
+	return o->prev ? FALSE : TRUE;
+}
+
 static HTMLFitType
 fit_line (HTMLObject *o,
 	  HTMLPainter *painter,
@@ -173,9 +207,7 @@ fit_line (HTMLObject *o,
 	slave = HTML_TEXT_SLAVE (o);
 	text  = HTML_TEXT (slave->owner);
 
-	/* if on begin of line (firstRun) and not on begin of para, remove beggining space */
-	if (firstRun && html_text_get_char (text, slave->posStart) == ' '
-	    && (o->prev != HTML_OBJECT (text) || HTML_OBJECT (text)->prev)) {
+	if (html_text_get_char (text, slave->posStart) == ' ' && could_remove_leading_space (slave, firstRun)) {
 		slave->posStart ++;
 		slave->posLen --;
 	}
@@ -252,7 +284,20 @@ get_length (HTMLObject *self)
 
 /* HTMLObject::draw() implementation.  */
 
-#ifdef GTKHTML_HAVE_PSPELL
+static gint
+get_ys (HTMLText *text, HTMLPainter *p)
+{
+	if (text->font_style & GTK_HTML_FONT_STYLE_SUBSCRIPT || text->font_style & GTK_HTML_FONT_STYLE_SUPERSCRIPT) {
+		gint height2;
+
+		height2 = (html_painter_calc_ascent (p, text->font_style, text->face)
+			   + html_painter_calc_descent (p, text->font_style, text->face)) >> 1;
+		return (text->font_style & GTK_HTML_FONT_STYLE_SUBSCRIPT) ? height2 : -height2;
+			
+	} else 
+		return 0;
+}
+
 static void
 draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
 {
@@ -271,7 +316,7 @@ draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
 
 			html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLSpellErrorColor)->color);
 			/* printf ("spell error: %s\n", HTML_TEXT (slave->owner)->text + off); */
-			html_painter_draw_spell_error (p, obj->x + tx, obj->y + ty,
+			html_painter_draw_spell_error (p, obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
 						       html_text_get_text (HTML_TEXT (slave->owner), slave->posStart),
 						       off, len);
 		}
@@ -280,7 +325,6 @@ draw_spell_errors (HTMLTextSlave *slave, HTMLPainter *p, gint tx, gint ty)
 		cur = cur->next;
 	}
 }
-#endif
 
 static void
 draw_normal (HTMLTextSlave *self,
@@ -299,7 +343,7 @@ draw_normal (HTMLTextSlave *self,
 	html_color_alloc (HTML_TEXT (self->owner)->color, p);
 	html_painter_set_pen (p, &HTML_TEXT (self->owner)->color->color);
 	html_painter_draw_text (p,
-				obj->x + tx, obj->y + ty, 
+				obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (self->owner), p),
 				html_text_get_text (HTML_TEXT (self->owner), self->posStart),
 				self->posLen);
 }
@@ -346,7 +390,8 @@ draw_highlighted (HTMLTextSlave *slave,
 	html_painter_fill_rect (p, obj->x + tx + offset_width, obj->y + ty - obj->ascent,
 				text_width, obj->ascent + obj->descent);
 	html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLHighlightTextColor)->color);
-	html_painter_draw_text (p, obj->x + tx + offset_width, obj->y + ty, text + unicode_offset_to_index (text, start), len);
+	html_painter_draw_text (p, obj->x + tx + offset_width, obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
+				text + unicode_offset_to_index (text, start), len);
 
 	/* Draw the non-highlighted part.  */
 
@@ -357,7 +402,7 @@ draw_highlighted (HTMLTextSlave *slave,
 
 	if (start > slave->posStart)
 		html_painter_draw_text (p,
-					obj->x + tx, obj->y + ty,
+					obj->x + tx, obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
 					text + unicode_offset_to_index (text, slave->posStart),
 					start - slave->posStart);
 
@@ -365,7 +410,8 @@ draw_highlighted (HTMLTextSlave *slave,
 
 	if (end < slave->posStart + slave->posLen)
 		html_painter_draw_text (p,
-					obj->x + tx + offset_width + text_width, obj->y + ty,
+					obj->x + tx + offset_width + text_width,
+					obj->y + ty + get_ys (HTML_TEXT (slave->owner), p),
 					text + unicode_offset_to_index (text, end),
 					slave->posStart + slave->posLen - end);
 }
@@ -394,16 +440,14 @@ draw (HTMLObject *o,
 	font_style = html_text_get_font_style (ownertext);
 
 	end = textslave->posStart + textslave->posLen;
-
 	if (owner->select_start + owner->select_length <= textslave->posStart
 	    || owner->select_start >= end) {
 		draw_normal (textslave, p, font_style, x, y, width, height, tx, ty);
 	} else {
 		draw_highlighted (textslave, p, font_style, x, y, width, height, tx, ty);
 	}
-#ifdef GTKHTML_HAVE_PSPELL
+
 	draw_spell_errors (textslave, p, tx ,ty);
-#endif
 }
 
 static gint
