@@ -22,12 +22,13 @@
 */
 
 /* The HTML Tokenizer */
+#include <config.h>
 #include <ctype.h>
 #include <gnome.h>
 #include "htmltokenizer.h"
 #include "htmlentity.h"
 
-#define TOKEN_BUFFER_SIZE (1 << 15)
+#define TOKEN_BUFFER_SIZE (1 << 10)
 
 typedef struct _HTMLBlockingToken HTMLBlockingToken;
 typedef struct _HTMLTokenBuffer   HTMLTokenBuffer;
@@ -72,7 +73,7 @@ struct _HTMLTokenizer {
 	gboolean style; /* Are we in a <style> block? */
 	gboolean script; /* Are we in a <script> block? */
 	gboolean textarea; /* Are we in a <textarea> block? */
-	gboolean pre; /* Are we in a <pre> block? */
+	gint     pre; /* Are we in a <pre> block? */
 	gboolean select; /* Are we in a <select> block? */
 	gboolean charEntity; /* Are we in an &... sequence? */
 
@@ -102,6 +103,7 @@ struct _HTMLTokenizer {
 	GList *blocking; /* Blocking tokens */
 
 	const gchar *searchFor;
+	gboolean utf8;
 };
 
 static const gchar *commentStart = "<!--";
@@ -191,7 +193,7 @@ html_tokenizer_new (void)
 	t->style = FALSE;
 	t->script = FALSE;
 	t->textarea = FALSE;
-	t->pre = FALSE;
+	t->pre = 0;
 	t->select = FALSE;
 	t->charEntity = FALSE;
 
@@ -223,6 +225,37 @@ html_tokenizer_destroy (HTMLTokenizer *tokenizer)
 	g_free (tokenizer);
 }
 
+gchar *
+html_tokenizer_peek_token (HTMLTokenizer *t)
+{
+	gchar *token;
+
+	g_assert (t->read_buf);
+
+	if (t->read_buf->used > t->read_pos) {
+		token = t->read_buf->data + t->read_pos;
+	} else {
+		GList *next;
+		HTMLTokenBuffer *buffer;
+
+		g_assert (t->read_cur);
+		g_assert (t->read_buf);
+
+		/* lookup for next buffer */
+		next = t->read_cur->next;
+		g_assert (next);
+
+		buffer = (HTMLTokenBuffer *) next->data;
+
+		g_return_val_if_fail (buffer->used != 0, NULL);
+
+		/* finally get first token */
+		token = buffer->data;
+	}
+	
+	return token;
+}
+	
 gchar *
 html_tokenizer_next_token (HTMLTokenizer *t)
 {
@@ -299,20 +332,22 @@ html_tokenizer_reset (HTMLTokenizer *t)
 	t->scriptCode = NULL;
 }
 
+static gint
+charset_is_utf8 (gchar *content_type)
+{
+	return content_type && strstr (content_type, "charset=utf-8") != NULL;
+}
+
 void
-html_tokenizer_begin (HTMLTokenizer *t)
+html_tokenizer_begin (HTMLTokenizer *t, gchar *content_type)
 {
 	html_tokenizer_reset (t);
 
-	t->buffer = g_malloc (10240);
-	t->dest = t->buffer;
-	t->size = 1000;
-
 	t->dest = t->buffer;
 	t->tag = FALSE;
 	t->pending = NonePending;
 	t->discard = NoneDiscard;
-	t->pre = FALSE;
+	t->pre = 0;
 	t->prePos = 0;
 	t->script = FALSE;
 	t->style = FALSE;
@@ -325,6 +360,16 @@ html_tokenizer_begin (HTMLTokenizer *t)
 	t->searchCount = 0;
 	t->title = FALSE;
 	t->charEntity = FALSE;
+	
+	t->utf8 = charset_is_utf8 (content_type);
+
+#if 0
+	if (t->utf8) 
+		g_warning ("Trying UTF-8");
+	else 
+		g_warning ("Trying ISO-8859-1");
+#endif
+
 }
 
 static void
@@ -347,6 +392,7 @@ html_tokenizer_end (HTMLTokenizer *t)
 	g_free (t->buffer);	
 
 	t->buffer = 0;
+	t->size = 0;
 
 	if (t->blocking) {
 		g_list_foreach (t->blocking, destroy_blocking, NULL);
@@ -385,7 +431,7 @@ html_tokenizer_append_token_buffer (HTMLTokenizer *t, gint min_size)
 	gint size = TOKEN_BUFFER_SIZE;
 
 	if (min_size > size)
-		size += min_size;
+		size = min_size + (min_size >> 2);
 
 	/* create new buffer and add it to list */
 	nb = html_token_buffer_new (size);
@@ -455,8 +501,8 @@ html_tokenizer_add_pending (HTMLTokenizer *t)
 	t->pending = NonePending;
 }
 
-gint
-g_unichar_to_utf8 (gint c, gchar *outbuf)
+static gint
+html_unichar_to_utf8 (gint c, gchar *outbuf)
 {
   size_t len = 0;
   gint first;
@@ -509,11 +555,10 @@ g_unichar_to_utf8 (gint c, gchar *outbuf)
 static void
 prepare_enough_space (HTMLTokenizer *t)
 {
-	/* I really do not understand it, but I added 5 for UTF-8 (Lauris) */
-	if ((t->dest - t->buffer + 5) > t->size) {
+	if ((t->dest - t->buffer + 32) > t->size) {
 		guint off = t->dest - t->buffer;
 
-		t->size  += 10240;
+		t->size  += (t->size >> 2) + 32;
 		t->buffer = g_realloc (t->buffer, t->size);
 		t->dest   = t->buffer + off;
 	}
@@ -599,7 +644,7 @@ in_entity (HTMLTokenizer *t, const gchar **src)
 	t->searchBuffer [t->searchCount + 1] = **src;
 	t->searchBuffer [t->searchCount + 2] = '\0';
 			
-			/* Check for &#0000 sequence */
+	/* Check for &#0000 sequence */
 	if (t->searchBuffer[2] == '#') {
 		if ((t->searchCount > 1) &&
 		    (!isdigit (**src)) &&
@@ -621,7 +666,7 @@ in_entity (HTMLTokenizer *t, const gchar **src)
 		}
 	}
 	else {
-				/* Check for &abc12 sequence */
+		/* Check for &abc12 sequence */
 		if (!isalnum (**src)) {
 			t->charEntity = FALSE;
 			if ((t->searchBuffer [t->searchCount + 1] == ';') ||
@@ -634,59 +679,9 @@ in_entity (HTMLTokenizer *t, const gchar **src)
 		}
 				
 	}
-			
-	switch (entityValue) {
-	case 139:
-		entityValue = 60;
-		break;
-	case 145:
-		entityValue = 96;
-		break;
-	case 146:
-		entityValue = 39;
-		break;
 
-	case 147:
-				//strcpy (t->searchBuffer+2, "ldquo");
-				//t->searchCount = 6;
-				//break;
-	case 148:
-				//strcpy(t->searchBuffer+2, "rdquo");
-				//searchCount = 6;
-		entityValue = 34;
-		break;
-	case 150:
-				//strcpy(t->searchBuffer+2, "ndash");
-				//t->searchCount = 6;
-				//break;
-	case 151:
-				//strcpy (t->searchBuffer+2, "mdash");
-				//t->searchCount = 6;
-		entityValue = 45;
-		break;
-	case 152:
-		entityValue = 126;
-		break;
-	case 155:
-		entityValue = 62;
-		break;
-	case 133:
-		strcpy (t->searchBuffer+2, "hellip");
-		t->searchCount = 7;
-		break;
-	case 149:
-		strcpy (t->searchBuffer+2, "bull");
-		t->searchCount = 5;
-		break;
-	case 153:
-		strcpy (t->searchBuffer+2, "trade");
-		t->searchCount = 6;
-		break;
-	default:;
-	}
-			
-	if (t->searchCount > 8) {
-				/* Ignore this sequence since it's too long */
+	if (t->searchCount > 9) {
+		/* Ignore this sequence since it's too long */
 		t->charEntity = FALSE;
 		memcpy (t->dest, t->searchBuffer + 1, t->searchCount);
 		t->dest += t->searchCount;
@@ -702,7 +697,7 @@ in_entity (HTMLTokenizer *t, const gchar **src)
 	else {
 		if(entityValue) {
 			/* Insert plain char */
-			t->dest += g_unichar_to_utf8 (entityValue, t->dest);
+			t->dest += html_unichar_to_utf8 (entityValue, t->dest);
 			if (t->pre) t->prePos++;
 			if (**src == ';') (*src)++;
 		}
@@ -762,7 +757,7 @@ in_tag (HTMLTokenizer *t, const gchar **src)
 		if (t->pending)
 			html_tokenizer_add_pending (t);
 		*(t->dest) = '<';     t->dest++;
-		*(t->dest)++ = **src; (*src)++;
+		*(t->dest) = **src;   t->dest++; (*src)++;
 		return;
 	}
 			
@@ -840,10 +835,10 @@ end_tag (HTMLTokenizer *t, const gchar **src)
 			
 	if (strncmp (t->buffer + 2, "pre", 3) == 0) {
 		t->prePos = 0;
-		t->pre = TRUE;
+		t->pre++;
 	}
 	else if (strncmp (t->buffer + 2, "/pre", 4) == 0) {
-		t->pre = FALSE;
+		t->pre--;
 	}
 	else if (strncmp (t->buffer + 2, "textarea", 8) == 0) {
 		t->textarea = TRUE;
@@ -1056,7 +1051,11 @@ in_plain (HTMLTokenizer *t, const gchar **src)
 	else if (t->pre) {
 		t->prePos++;
 	}
-	t->dest += g_unichar_to_utf8 ((guchar) **src, t->dest);
+	if (t->utf8)
+		*(t->dest)++ = **src;
+	else 
+		t->dest += html_unichar_to_utf8 ((guchar) **src, t->dest);
+	
 	(*src)++;
 }
 
@@ -1065,9 +1064,6 @@ html_tokenizer_write (HTMLTokenizer *t, const gchar *string, size_t size)
 {
 	const gchar *src = string;
 
-	if (!t->buffer)
-		html_tokenizer_begin(t);
-
 	while ((src - string) < size) {
 		prepare_enough_space (t);
 		
