@@ -19,6 +19,9 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include "htmlcursor.h"
+#include "htmlengine.h"
 #include "htmlundo.h"
 
 
@@ -91,9 +94,7 @@ do_action (HTMLUndo *undo,
 	first = *action_list;
 	*action_list = (*action_list)->next;
 
-	if (*action_list == NULL)
-		g_warning ("No more actions in list!");
-	else
+	if (*action_list)
 		(*action_list)->prev = NULL;
 
 	first->next = NULL;
