@@ -30,6 +30,7 @@
 #include "htmlurl.h"
 #include "htmlengine.h"
 #include "gtkhtml-embedded.h"
+#include "gtkhtml-properties.h"
 
 #include "gtkhtmldebug.h"
 
@@ -1033,10 +1034,10 @@ main (gint argc, gchar *argv[])
 	GtkWidget *app, *bar;
 	GtkWidget *html_widget;
 	GtkWidget *scrolled_window;
-	GdkColor   bgColor = {0, 0xdfff, 0xdfff, 0xffff};
+	GdkColor   bgColor = {0, 0xc7ff, 0xc7ff, 0xc7ff};
 	poptContext ctx;
 #ifdef GTKHTML_HAVE_GCONF
-	GConfError  *gconf_error  = NULL;
+	GError  *gconf_error  = NULL;
 #endif
 
 #ifdef MEMDEBUG
@@ -1048,7 +1049,7 @@ main (gint argc, gchar *argv[])
 #ifdef GTKHTML_HAVE_GCONF
 	if (!gconf_init (argc, argv, &gconf_error)) {
 		g_assert (gconf_error != NULL);
-		g_error ("GConf init failed:\n  %s", gconf_error->str);
+		g_error ("GConf init failed:\n  %s", gconf_error->message);
 		return FALSE;
 	}
 #endif
